"""
Struktura uczelni.
"""
from datetime import date, timedelta

from autoslug import AutoSlugField
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import CASCADE
from django.db.models.functions import Coalesce
from django.db.models.query_utils import Q
from django.urls.base import reverse
from mptt.fields import TreeForeignKey
from mptt.managers import TreeManager
from mptt.models import MPTTModel

from .uczelnia import Uczelnia
from .wydzial import Wydzial

from django.contrib.postgres.search import SearchVectorField as VectorField

from django.utils import timezone

from bpp.models import ModelZAdnotacjami, ModelZPBN_UID
from bpp.models.abstract import ModelZPBN_ID
from bpp.models.autor import Autor, Autor_Jednostka
from bpp.util import FulltextSearchMixin, safe_html

SORTUJ_RECZNIE = ("kolejnosc", "nazwa")
SORTUJ_ALFABETYCZNIE = ("nazwa",)


class JednostkaManager(FulltextSearchMixin, TreeManager):
    def create(self, *args, **kw):
        if "wydzial" in kw and not ("uczelnia" in kw or "uczelnia_id" in kw):
            # Kompatybilność wsteczna, z czasów, gdy nie było metryczki historycznej
            # dla obecności jednostki w wydziałach
            kw["uczelnia"] = kw["wydzial"].uczelnia
        return super().create(*args, **kw)

    def get_default_ordering(self):
        uczelnia = Uczelnia.objects.get_default()

        ordering = SORTUJ_RECZNIE
        if uczelnia is None:
            ordering = SORTUJ_ALFABETYCZNIE
        else:
            if uczelnia.sortuj_jednostki_alfabetycznie:
                ordering = SORTUJ_ALFABETYCZNIE

        return ordering

    def get_queryset(self, *args, **kwargs):
        return super().get_queryset(*args, **kwargs).select_related("wydzial")

    def widoczne(self):
        "Jednostki widoczne (nie-ukryte)"
        return self.filter(widoczna=True)

    def publiczne(self):
        """Jednostki widoczne publicznie"""
        return self.widoczne().filter(aktualna=True)


class Jednostka(ModelZAdnotacjami, ModelZPBN_ID, ModelZPBN_UID, MPTTModel):
    parent = TreeForeignKey(
        "self",
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        related_name="children",
        verbose_name="Jednostka nadrzędna",
    )

    uczelnia = models.ForeignKey(
        Uczelnia,
        CASCADE,
        # Jeżeli dam tu rozsądny default, żeby w adminie się wyświetlało prawidłowo,
        # to z kolei wysiądzie mi cała masa testów, korzystająca z model_bakery
        # i tworząca obiekt 'Uczelnia' na poczekaniu (pole nie może być NULL).
        # Zatem, zostawiamy to wyłączone i w adminie ustawimy wartośći inicjalne.
        # default=lambda: Uczelnia.objects.first()
    )

    wydzial = models.ForeignKey(
        Wydzial, CASCADE, verbose_name="Wydział", blank=True, null=True
    )
    aktualna = models.BooleanField(
        default=False,
        help_text="""Jeżeli dana jednostka wchodzi w struktury wydziału
    (czyli jej obecność w strukturach wydziału nie została zakończona z określoną datą), to pole to będzie miało
    wartość 'PRAWDA'.""",
    )

    nazwa = models.CharField(max_length=512, unique=True)
    skrot = models.CharField("Skrót", max_length=128, unique=True)
    opis = models.TextField(blank=True, null=True)
    pokazuj_opis = models.BooleanField(
        default=True,
        help_text="Gdy to pole jest zaznaczone, system wyświetli pole 'Opis' na podstronie jednostki.",
    )
    slug = AutoSlugField(populate_from="nazwa", unique=True)

    widoczna = models.BooleanField(default=True, db_index=True)
    wchodzi_do_raportow = models.BooleanField(
        "Wchodzi do raportów",
        default=True,
        db_index=True,
        help_text="Jeżeli odznaczone, prace z jednostki nie sumują się w rankingu autorów.",
    )
    email = models.EmailField("E-mail", max_length=128, blank=True, null=True)
    www = models.URLField("WWW", max_length=1024, blank=True, null=True)

    pbn_uid = models.ForeignKey(
        "pbn_api.Institution",
        verbose_name="Odpowiednik w PBN",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )

    skupia_pracownikow = models.BooleanField(
        "Skupia pracowników",
        default=True,
        help_text="""Ta jednostka skupia osoby będące faktycznymi pracownikami uczelni. Odznacz dla jednostek
         typu 'Studenci', 'Doktoranci', 'Pracownicy emerytowani' itp. Odznacz dla 'obcych' jednostek. """,
        db_index=True,
    )

    zarzadzaj_automatycznie = models.BooleanField(
        "Zarządzaj automatycznie",
        default=True,
        help_text="""Jednostka ta będzie dowolnie modyfikowana przez procedury importujace dane z zewnętrznych
        systemów informatycznych""",
    )

    search = VectorField(blank=True, null=True)

    kolejnosc = models.PositiveIntegerField(default=0, blank=False, null=False)

    objects = JednostkaManager()

    class Meta:
        verbose_name = "jednostka"
        verbose_name_plural = "jednostki"
        ordering = ["kolejnosc", "nazwa"]
        app_label = "bpp"

    class MPTTMeta:
        order_insertion_by = ["kolejnosc", "nazwa"]

    def get_absolute_url(self):
        return reverse("bpp:browse_jednostka", args=(self.slug,))

    def __str__(self):
        ret = self.nazwa

        try:
            wydzial = self.wydzial
        except (ValueError, TypeError, Wydzial.DoesNotExist):  # TODO catch-all
            wydzial = None

        if wydzial is not None:
            ret += " (%s)" % self.wydzial.skrot

        return ret

    def dodaj_autora(
        self, autor, funkcja=None, rozpoczal_prace=None, zakonczyl_prace=None
    ):
        ret = Autor_Jednostka.objects.create(
            autor=autor,
            jednostka=self,
            funkcja=funkcja,
            rozpoczal_prace=rozpoczal_prace,
            zakonczyl_prace=zakonczyl_prace,
        )
        # Odśwież obiekt - pobierz ewentualną zmiane pola 'aktualna_jednostka', obsługiwaną
        # przez trigger bazodanowy (migracja 0046)
        autor.refresh_from_db()
        return ret

    zatrudnij = dodaj_autora

    #
    # "Stare (przed-2022)" procedury wyświetlające autorów obecnie przypisanych do tej jednostki
    #

    def obecni_autorzy(self):
        dzis = timezone.now().date()

        return Autor.objects.filter(
            Q(autor_jednostka__zakonczyl_prace__gte=dzis)
            | Q(autor_jednostka__zakonczyl_prace=None),
            Q(autor_jednostka__rozpoczal_prace__lte=dzis)
            | Q(autor_jednostka__rozpoczal_prace=None),
            autor_jednostka__jednostka=self,
        ).distinct()

    def autorzy_na_strone_jednostki(self):
        """ "Stara" funkcja, wyswietlajaca autorow przypisanych do tej jednostki,
        w przypadku, gdy nikt nie ma 'Podstawowe miejsce pracy' ustawione na TRUE"""
        return self.obecni_autorzy().filter(pokazuj=True)

    #
    # "Nowe (2022)" procedury wyświetlające aktualnych autorów (ta jednostka ma przypisanie
    # gdzie podstawowe_miejsce_pracy=True) oraz poprzednich współpracowników
    #

    def aktualni_autorzy(self):
        return (
            Autor_Jednostka.objects.filter(
                podstawowe_miejsce_pracy=True, jednostka=self
            )
            .values_list("autor")
            .distinct()
        )

    def pracownicy(self):
        """Autorzy, którzy tą jednostkę mają wpisani jako AKTUALNA -- czyli
        aktualni pracownicy, obecni pracownicy"""
        return Autor.objects.filter(pk__in=self.aktualni_autorzy(), pokazuj=True)

    def wspolpracowali(self):
        """Autorzy, którzy popełnili jakiekolwiek prace z afiliacją na tę jednostkę,
        nie będący autorami z  funkcji 'pracownicy'"""
        from bpp.models.cache import Autorzy

        return Autor.objects.filter(
            pk__in=Autorzy.objects.filter(jednostka=self)
            .exclude(autor_id__in=self.aktualni_autorzy())
            .values_list("autor")
        )

    def kierownik(self):
        try:
            return self.obecni_autorzy().get(
                autor_jednostka__funkcja__nazwa="kierownik"
            )
        except Autor.DoesNotExist:
            return None

    def prace_w_latach(self):
        from bpp.models.cache import Rekord

        return (
            Rekord.objects.prace_jednostki(self)
            .values_list("rok", flat=True)
            .distinct()
            .order_by("rok")
        )

    def przypisania(self):
        return Jednostka_Wydzial.objects.filter(jednostka_id=self.pk).order_by("od")

    def przypisania_dla_czasokresu(self, od, do):
        return Jednostka_Wydzial.objects.dla_czasokresu(od=od, do=do).filter(
            jednostka_id=self.pk
        )

    def przypisanie_dla_dnia(self, data):
        return self.przypisania_dla_czasokresu(data, data).first()

    def wydzial_dnia(self, data):
        try:
            return self.przypisanie_dla_dnia(data).wydzial
        except AttributeError:
            return

    def clean(self):
        if self.opis:
            self.opis = safe_html(self.opis)


class Jednostka_Wydzial_Manager(models.Manager):
    def od_do_not_null(self):
        return self.get_queryset().annotate(
            od_not_null=Coalesce("od", date(1, 1, 1)),
            do_not_null=Coalesce("do", date(9999, 12, 31)),
        )

    def dla_czasokresu(self, od, do):
        return self.od_do_not_null().filter(
            od_not_null__lte=do or date(9999, 12, 31),
            do_not_null__gte=od or date(1, 1, 1),
        )

    def wyczysc_przypisania(self, jednostka, parent_od=None, parent_do=None):
        parent_do_not_null = parent_do or date(9999, 12, 31)

        for jw in (
            self.dla_czasokresu(parent_od, parent_do)
            .filter(jednostka_id=jednostka.pk)
            .order_by("od_not_null")
        ):
            od = jw.od or date(1, 1, 1)
            do = jw.do or date(9999, 12, 31)

            # Jeżeli zakres kończy przed parent.do, to nie ma prawa
            # być takiej sytuacji, bo funkcja przypisania_dla_czasokresu
            # ma nie zwracać takich zakresów. Ma prawo kończyć się w dniu parent.do
            # ale nie ma się kończyć przed
            assert (
                do >= parent_od
            ), "To nie powinno się zdarzyć. Funkcja przypisania_dla_czasokresu działa niepoprawnie"

            # Jeżeli zakres zaczyna się za parent.do, to nie ma prawa
            # być takiej sytuacji, bo funkcja przypisania_dla_czasokresu
            # ma nie zwracać takich zakresów. Ma prawo zaczynać się w dniu parent.od
            # ale nie ma prawa zaczynać się za:
            assert (
                od <= parent_do_not_null
            ), "To nie powinno się zdarzyć. Funkcja przypisania_dla_czasokresu działa niepoprawnie"

            # Jeżeli zakres zaczyna się przed parent.od i kończy wewnątrz parent
            #
            #      +---+
            #          |........|
            #
            #      +-------+
            #          |........|
            #
            #      +------------+
            #          |........|

            if od < parent_od and do <= parent_do_not_null:
                jw.do = parent_od - timedelta(days=1)
                jw.save()
                continue

            # Jeżeli zakres zaczyna się przed parent.od i kończy za parent
            #
            #      +---------------+
            #          |........|
            if od < parent_od and do > parent_do:
                old_do = jw.do
                new_do = parent_od - timedelta(days=1)
                new_od = parent_do + timedelta(days=1)

                jw.do = new_do
                jw.save()

                Jednostka_Wydzial.objects.create(
                    jednostka=jw.jednostka, wydzial=jw.wydzial, od=new_od, do=old_do
                )
                continue

            # Jeżeli zakres zaczyna się na lub po parent.od i kończy wewnątrz
            #
            #          +----+
            #          |........|
            #
            #          +--------+
            #          |........|
            #
            #             +-----+
            #          |........|
            if od >= parent_od and do <= parent_do:
                jw.delete()
                continue

            # Jeżeli zakres zaczyna się na lub po parent.od i kończy po parent.do
            #
            #
            #               +-----------+
            #          |........|
            #
            #          +--------------+
            #          |........|
            if od >= parent_od and do > parent_do:
                jw.od = parent_do + timedelta(days=1)
                jw.save()
                continue


class Jednostka_Wydzial(models.Model):
    jednostka = models.ForeignKey(Jednostka, CASCADE)
    wydzial = models.ForeignKey(Wydzial, CASCADE)
    od = models.DateField(null=True, blank=True)
    do = models.DateField(null=True, blank=True)

    objects = Jednostka_Wydzial_Manager()

    class Meta:
        verbose_name = "powiązanie jednostka-wydział"
        verbose_name_plural = "powiązania jednostka-wydział"
        ordering = ("-od",)

    def __str__(self):
        return f"{self.jednostka} - {self.wydzial} ({self.od}, {self.do})"

    def clean(self):
        try:
            self.wydzial
        except (ValueError, TypeError, Wydzial.DoesNotExist):
            raise ValidationError({"wydzial": "Określ wydział"})

        if self.wydzial.uczelnia_id != self.jednostka.uczelnia_id:
            raise ValidationError(
                {"wydzial": "Uczelnia dla wydziału i jednostki musi być identyczna."}
            )

        if self.od is not None and self.do is not None:
            if self.od >= self.do:
                raise ValidationError(
                    {
                        "od": "Wartość w polu 'Od' musi byc mniejsza, niż wartość w polu 'Do'.",
                        "do": "Wartosć w polu 'Do' musi być większa, niż wartość w polu 'Od'.",
                    }
                )

        if self.pk:
            try:
                old = Jednostka_Wydzial.objects.get(pk=self.pk)
                if old.jednostka_id != self.jednostka_id:
                    raise ValidationError(
                        {
                            "jednostka": "Zmiana ID jednostki dla tych obiektów nie jest obsługiwana."
                        }
                    )
            except Jednostka_Wydzial.DoesNotExist:
                pass

        # Sprawdz zakres dat
        cnt = (
            Jednostka_Wydzial.objects.dla_czasokresu(self.od, self.do)
            .filter(jednostka_id=self.jednostka_id)
            .exclude(id=self.id)
            .count()
        )

        if cnt:
            msg = """rekord z podobnym lub nakładającym się zakresem dat już istnieje w bazie danych.
            Nie możesz dodać nakładających się zakresów dat. Jeżeli próbujesz dokonać kilku zmian jednocześnie, ten
            komunikat może pojawić się również, gdy taka sytuacja nie zachodzi. Wówczas zmień tylko jeden
            rekord jednoczasowo - rozbij zmianę na kilka pojedynczych działań."""
            raise ValidationError({"od": msg})

        # Sprawdź, czy pole "do" nie zawiera daty w przyszłości
        d = self.do or date(1, 1, 1)
        if d >= date.today():
            raise ValidationError(
                {
                    "do": 'Data w polu "Do" nie może być większa lub równa, niż data aktualna (dzisiejsza).'
                }
            )



# Generated by Django 3.0.14 on 2022-02-27 20:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0312_przypieta"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="wydawnictwo_ciagle_zewnetrzna_baza_danych",
            options={
                "verbose_name": "powiązanie wyd. ciągłego z zewn. bazą danych",
                "verbose_name_plural": "powiązania wyd. ciągłych z zewn. bazami danych",
            },
        ),
        migrations.AlterModelOptions(
            name="wydawnictwo_zwarte_zewnetrzna_baza_danych",
            options={
                "verbose_name": "powiązanie wyd. zwartego z zewn. bazami danych",
                "verbose_name_plural": "powiązania wyd. zwartych z zewn. bazami danych",
            },
        ),
    ]



from notifications.mixins import (
    ChannelSubscriberMixin,
    ChannelSubscriberSingleObjectMixin,
)
import pytest


@pytest.mark.django_db
def test_ChannelSubscriberMixin_subscribe_to(wydawnictwo_zwarte):
    x = ChannelSubscriberMixin()
    x.subscribe_to(wydawnictwo_zwarte)
    assert len(x._subscribed_to) == 1


@pytest.mark.django_db
def test_ChannelSubscriberMixin_get_context_data():
    class Foo:
        def get_context_data(self):
            return {"a": "B"}

    class Bar(ChannelSubscriberMixin, Foo):
        pass

    x = Bar()
    res = x.get_context_data()
    assert "extraChannels" in res.keys()


@pytest.mark.django_db
def test_ChannelSubscriberSingleObjectMixin(wydawnictwo_zwarte):
    class Foo:
        def get_object(self):
            return wydawnictwo_zwarte

    class Bar(ChannelSubscriberSingleObjectMixin, Foo):
        pass

    x = Bar()
    assert x.get_object() == wydawnictwo_zwarte
    assert len(x._subscribed_to) == 1



# Generated by Django 3.0.11 on 2021-01-30 13:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("raport_slotow", "0006_auto_20210125_2330"),
    ]

    operations = [
        migrations.AddField(
            model_name="raportslotowuczelnia",
            name="pokazuj_zerowych",
            field=models.BooleanField(
                default=False,
                help_text="Autor z zerowym slotem to autor, który ma na zadany okres czasu zadeklarowane dyscypliny,\n        ale nie posiada żadnych punktowanych prac. ",
                verbose_name="Dołączaj autorów z zerowymi slotami",
            ),
        ),
    ]



# Generated by Django 3.0.9 on 2020-08-12 17:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("bpp", "0221_merge_20200806_0851"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="grant",
            options={"verbose_name": "grant", "verbose_name_plural": "granty"},
        ),
        migrations.RemoveField(model_name="praca_doktorska", name="grant",),
        migrations.RemoveField(model_name="praca_habilitacyjna", name="grant",),
        migrations.RemoveField(model_name="wydawnictwo_ciagle", name="grant",),
        migrations.RemoveField(model_name="wydawnictwo_zwarte", name="grant",),
        migrations.CreateModel(
            name="Grant_Rekordu",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("object_id", models.PositiveIntegerField()),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
                (
                    "grant",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="bpp.Grant"
                    ),
                ),
            ],
        ),
    ]



import json
import re

import pytest
from bs4 import BeautifulSoup

from django.contrib.contenttypes.models import ContentType

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from model_bakery import baker
from multiseek.logic import EQUAL, EQUAL_FEMALE, EQUAL_NONE
from multiseek.views import MULTISEEK_SESSION_KEY

from miniblog.models import Article

from bpp.models import Rekord, Wydawnictwo_Zwarte
from bpp.views.browse import BuildSearch, PracaViewBySlug


def test_buildSearch(settings):
    dct = {
        "zrodlo": [
            1,
        ],
        "typ": [
            1,
        ],
        "rok": [
            2013,
        ],
        "jednostka": [
            1,
        ],
        "autor": [
            1,
        ],
    }

    class mydct(dict):
        def getlist(self, value):
            return self.get(value)

    class request:
        POST = mydct(dct)
        META = {}
        session = {}

        def build_absolute_uri(self, *args, **kw):
            return "/absolute/uri"

    settings.LANGUAGE_CODE = "en"
    tbs = BuildSearch()
    tbs.request = request()
    tbs.post(request)

    expected = {
        "form_data": [
            None,
            {
                "field": "\u0179r\xf3d\u0142o",
                "operator": str(EQUAL_NONE),
                "prev_op": None,
                "value": 1,
            },
            {
                "field": "Nazwisko i imi\u0119",
                "operator": str(EQUAL_NONE),
                "prev_op": "and",
                "value": 1,
            },
            {
                "field": "Typ rekordu",
                "operator": str(EQUAL),
                "prev_op": "and",
                "value": 1,
            },
            {
                "field": "Jednostka",
                "operator": str(EQUAL_FEMALE),
                "prev_op": "and",
                "value": 1,
            },
            {"field": "Rok", "operator": str(EQUAL), "prev_op": "and", "value": 2013},
        ]
    }

    assert json.loads(request.session[MULTISEEK_SESSION_KEY]) == expected


pattern = re.compile("Strona WWW")


def nastepna_komorka_po_strona_www(dokument):
    soup = BeautifulSoup(dokument, "html.parser")
    return soup.find("th", text=pattern).parent.find("td").text.strip()


@pytest.mark.django_db
def test_darmowy_platny_dostep_www_wyswietlanie(client, wydawnictwo_ciagle, denorms):
    wydawnictwo_ciagle.www = ""
    wydawnictwo_ciagle.public_www = ""
    wydawnictwo_ciagle.save()

    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
                wydawnictwo_ciagle.pk,
            ),
        ),
        follow=True,
    )
    val = nastepna_komorka_po_strona_www(res.content)
    assert val == "Brak danych"

    wydawnictwo_ciagle.www = "platny"
    wydawnictwo_ciagle.public_www = ""
    wydawnictwo_ciagle.save()

    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
                wydawnictwo_ciagle.pk,
            ),
        ),
        follow=True,
    )
    val = nastepna_komorka_po_strona_www(res.content)
    assert val == "platny"

    wydawnictwo_ciagle.www = ""
    wydawnictwo_ciagle.public_www = "darmowy"
    wydawnictwo_ciagle.save()
    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
                wydawnictwo_ciagle.pk,
            ),
        ),
        follow=True,
    )
    val = nastepna_komorka_po_strona_www(res.content)
    assert val == "darmowy"

    wydawnictwo_ciagle.www = "jezeli sa oba ma byc darmowy"
    wydawnictwo_ciagle.public_www = "darmowy"
    wydawnictwo_ciagle.save()
    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
                wydawnictwo_ciagle.pk,
            ),
        ),
        follow=True,
    )
    val = nastepna_komorka_po_strona_www(res.content)
    assert val == "darmowy"


@pytest.mark.django_db
def test_artykuly(uczelnia, client):
    res = client.get(reverse("bpp:browse_uczelnia", args=(uczelnia.slug,)))
    assert res.status_code == 200

    TYTUL = "Tytul testowego artykulu"

    a = Article.objects.create(
        title=TYTUL, article_body="456", status=Article.STATUS.draft, slug="1"
    )

    res = client.get(reverse("bpp:browse_uczelnia", args=(uczelnia.slug,)))
    assert TYTUL.encode("utf-8") not in res.content

    a.status = Article.STATUS.published
    a.save()

    res = client.get(reverse("bpp:browse_uczelnia", args=(uczelnia.slug,)))
    assert TYTUL.encode("utf-8") in res.content


@pytest.mark.django_db
def test_artykul_ze_skrotem(uczelnia, client):
    a = Article.objects.create(
        title="123",
        article_body="456\n<!-- tutaj -->\nTego ma nie byc",
        status=Article.STATUS.published,
        slug="1",
    )

    res = client.get(reverse("bpp:browse_uczelnia", args=(uczelnia.slug,)))
    assert b"Tego ma nie byc" not in res.content
    assert "więcej" in res.rendered_content

    res = client.get(reverse("bpp:browse_artykul", args=(uczelnia.slug, a.slug)))
    assert b"Tego ma nie byc" in res.content


@pytest.mark.django_db
def test_browse_snip_visible(client, uczelnia, wydawnictwo_ciagle):
    wydawnictwo_ciagle.punktacja_snip = 50
    wydawnictwo_ciagle.save()

    uczelnia.pokazuj_punktacja_snip = True
    uczelnia.save()

    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
                wydawnictwo_ciagle.pk,
            ),
        ),
        follow=True,
    )

    assert b"SNIP" in res.content


@pytest.mark.django_db
def test_browse_snip_invisible(client, uczelnia, wydawnictwo_ciagle):
    wydawnictwo_ciagle.punktacja_snip = 50
    wydawnictwo_ciagle.save()

    uczelnia.pokazuj_punktacja_snip = False
    uczelnia.save()

    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get(app_label="bpp", model="wydawnictwo_ciagle").pk,
                wydawnictwo_ciagle.pk,
            ),
        ),
        follow=True,
    )

    assert b"SNIP" not in res.content


@pytest.mark.django_Db
def test_browse_praca_wydawnictwa_powiazane(wydawnictwo_zwarte, client, denorms):
    # Testuj, czy rozdziały pokazują się w wydawnictwach powiązanych
    baker.make(
        Wydawnictwo_Zwarte,
        wydawnictwo_nadrzedne=wydawnictwo_zwarte,
        tytul_oryginalny="Roz 1",
        strony="asdoifj 55-34 oaijsdfo",
    )
    baker.make(
        Wydawnictwo_Zwarte,
        wydawnictwo_nadrzedne=wydawnictwo_zwarte,
        tytul_oryginalny="Roz 2",
        strony="IXIXI 22-50",
    )

    denorms.flush()

    wydawnictwo_zwarte.refresh_from_db()

    url = reverse("bpp:browse_praca_by_slug", args=(wydawnictwo_zwarte.slug,))
    res = client.get(url)

    assert b"Rekordy powi" in res.content
    x1 = res.content.find(b"Roz 1")
    x2 = res.content.find(b"Roz 2")

    # Sortujemy po polu "strony", jeden ma byc pozniej, drugi wczesniej:
    assert x2 < x1


@pytest.mark.django_db
def test_praca_tabela_no_pmc_id(wydawnictwo_zwarte, client):
    wydawnictwo_zwarte.pmc_id = None
    wydawnictwo_zwarte.save()

    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get_for_model(wydawnictwo_zwarte).pk,
                wydawnictwo_zwarte.pk,
            ),
        )
    )

    assert b"https://www.ncbi.nlm.nih.gov/pmc/" not in res.content


@pytest.mark.django_db
def test_praca_tabela_pmc_id(wydawnictwo_zwarte, client):
    wydawnictwo_zwarte.pmc_id = "123"
    wydawnictwo_zwarte.save()

    res = client.get(
        reverse(
            "bpp:browse_praca",
            args=(
                ContentType.objects.get_for_model(wydawnictwo_zwarte).pk,
                wydawnictwo_zwarte.pk,
            ),
        ),
        follow=True,
    )

    assert b"https://www.ncbi.nlm.nih.gov/pmc/" in res.content


def test_PracaView_ukrywanie_statusy_anonim(
    client, uczelnia, przed_korekta, wydawnictwo_zwarte_przed_korekta
):
    url = reverse(
        "bpp:browse_praca",
        args=(
            ContentType.objects.get_for_model(wydawnictwo_zwarte_przed_korekta).pk,
            wydawnictwo_zwarte_przed_korekta.pk,
        ),
    )

    res = client.get(url, follow=True)
    assert res.status_code == 200

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    res = client.get(url, follow=True)
    assert res.status_code == 403


def test_PracaView_ukrywanie_statusy_admin(
    admin_client, uczelnia, przed_korekta, wydawnictwo_zwarte_przed_korekta
):
    url = reverse(
        "bpp:browse_praca",
        args=(
            ContentType.objects.get_for_model(wydawnictwo_zwarte_przed_korekta).pk,
            wydawnictwo_zwarte_przed_korekta.pk,
        ),
    )

    res = admin_client.get(url, follow=True)
    assert res.status_code == 200

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)

    res = admin_client.get(url, follow=True)
    assert res.status_code == 200


def test_PracaViewBySlug_get_object(wydawnictwo_zwarte, denorms):
    denorms.rebuildall("Wydawnictwo_Zwarte")
    wydawnictwo_zwarte.refresh_from_db()
    r = Rekord.objects.get_for_model(wydawnictwo_zwarte)
    o = PracaViewBySlug(kwargs=dict(slug=wydawnictwo_zwarte.slug)).get_object()
    assert o == r

    o = PracaViewBySlug(
        kwargs=dict(
            slug=f"stary-slug-"
            f"{ContentType.objects.get_for_model(wydawnictwo_zwarte).pk}-{wydawnictwo_zwarte.pk}"
        )
    ).get_object()
    assert o == r


def test_PracaViewMixin_redirect(wydawnictwo_zwarte, rf, admin_user, denorms):
    denorms.rebuildall("Wydawnictwo_Zwarte")
    req = rf.get("/")
    req.user = admin_user
    res = PracaViewBySlug(
        kwargs=dict(
            slug=f"zlys-lug-{ContentType.objects.get_for_model(wydawnictwo_zwarte).pk}-{wydawnictwo_zwarte.pk}"
        )
    ).get(req)
    assert res.status_code == 302
    assert res.url.find("/bpp/rekord/Wydawnictwo-Zwarte") == 0



import os
import subprocess

from django.contrib.staticfiles import utils as django_utils
from django.contrib.staticfiles.finders import FileSystemFinder
from django.core.files.storage import FileSystemStorage
from django.conf import settings

try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict


def yarn_add():
    yarn_executable_path = getattr(settings, 'YARN_EXECUTABLE_PATH', 'yarn')
    command = [yarn_executable_path, 'add', '--prefix=' + get_yarn_root_path()]
    proc = subprocess.Popen(
        command,
        env={'PATH': os.environ.get('PATH')},
    )
    proc.wait()


def get_yarn_root_path():
    return getattr(settings, 'YARN_ROOT_PATH', '.')


def flatten_patterns(patterns):
    if patterns is None:
        return None
    return [
        os.path.join(module, module_pattern)
        for module, module_patterns in list(patterns.items())
        for module_pattern in module_patterns
    ]


def may_contain_match(directory, patterns):
    return any(pattern.startswith(directory) for pattern in patterns)


def get_files(storage, match_patterns='*', ignore_patterns=None, location=''):
    if ignore_patterns is None:
        ignore_patterns = []
    if match_patterns is None:
        match_patterns = []

    directories, files = storage.listdir(location)
    for fn in files:
        if django_utils.matches_patterns(fn, ignore_patterns):
            continue
        if location:
            fn = os.path.join(location, fn)
        if not django_utils.matches_patterns(fn, match_patterns):
            continue
        yield fn
    for dir in directories:
        if django_utils.matches_patterns(dir, ignore_patterns):
            continue
        if location:
            dir = os.path.join(location, dir)
        if may_contain_match(dir, match_patterns) or django_utils.matches_patterns(dir, match_patterns):
            for fn in get_files(storage, match_patterns, ignore_patterns, dir):
                yield fn


class YarnFinder(FileSystemFinder):
    def __init__(self, apps=None, *args, **kwargs):
        self.node_modules_path = get_yarn_root_path()
        self.destination = getattr(settings, 'YARN_STATIC_FILES_PREFIX', '')
        self.cache_enabled = getattr(settings, 'YARN_FINDER_USE_CACHE', True)
        self.cached_list = None

        self.match_patterns = flatten_patterns(getattr(settings, 'YARN_FILE_PATTERNS', None)) or ['*']
        self.locations = [(self.destination, os.path.join(self.node_modules_path, 'node_modules'))]
        self.storages = OrderedDict()

        filesystem_storage = FileSystemStorage(location=self.locations[0][1])
        filesystem_storage.prefix = self.locations[0][0]
        self.storages[self.locations[0][1]] = filesystem_storage

    def find(self, path, all=False):
        relpath = os.path.relpath(path, self.destination)
        if not django_utils.matches_patterns(relpath, self.match_patterns):
            return []
        return super(YarnFinder, self).find(path, all=all)

    def list(self, ignore_patterns=None):  # TODO should be configurable, add setting
        """List all files in all locations."""
        if self.cache_enabled:
            if self.cached_list is None:
                self.cached_list = list(self._make_list_generator(ignore_patterns))
            return self.cached_list
        return self._make_list_generator(ignore_patterns)

    def _make_list_generator(self, ignore_patterns=None):
        for prefix, root in self.locations:
            storage = self.storages[root]
            for path in get_files(storage, self.match_patterns, ignore_patterns):
                yield path, storage


# Generated by Django 3.0.14 on 2021-10-03 18:23

from django.db import migrations, models

import django.contrib.postgres.fields
import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0299_wydawnictwo_ciagle_denorm"),
    ]

    operations = [
        migrations.AddField(
            model_name="patent",
            name="cached_punkty_dyscyplin",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                blank=True, editable=False, null=True
            ),
        ),
        migrations.AlterField(
            model_name="patent",
            name="opis_bibliograficzny_autorzy_cache",
            field=django.contrib.postgres.fields.ArrayField(
                base_field=models.TextField(),
                blank=True,
                editable=False,
                null=True,
                size=None,
            ),
        ),
        migrations.AlterField(
            model_name="patent",
            name="opis_bibliograficzny_cache",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="patent",
            name="opis_bibliograficzny_zapisani_autorzy_cache",
            field=models.TextField(blank=True, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name="patent",
            name="slug",
            field=models.SlugField(
                blank=True, editable=False, max_length=400, null=True, unique=True
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2022-09-10 14:54

import django.db.models.deletion
import django.db.models.manager
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0327_uczelnia_pokazuj_formularz_zglaszania_publikacji"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("zglos_publikacje", "0011_auto_20220910_1646"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="obslugujacy_zgloszenia_wydzialow",
            managers=[
                ("manager", django.db.models.manager.Manager()),
            ],
        ),
        migrations.AlterField(
            model_name="obslugujacy_zgloszenia_wydzialow",
            name="user",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
                verbose_name="Użytkownik",
            ),
        ),
        migrations.AlterField(
            model_name="obslugujacy_zgloszenia_wydzialow",
            name="wydzial",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.wydzial",
                verbose_name="Wydział",
            ),
        ),
        migrations.AlterUniqueTogether(
            name="obslugujacy_zgloszenia_wydzialow",
            unique_together={("user", "wydzial")},
        ),
    ]



# Generated by Django 2.2.10 on 2020-03-15 21:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0202_fix_autorzy_bez_nazwisk"),
        ("bpp", "0204_cache_punktacja_autora_query_view"),
    ]

    operations = []



from mptt.admin import DraggableMPTTAdmin

from ..models import Charakter_Formalny  # Publikacja_Habilitacyjna
from .core import BaseBppAdminMixin, RestrictDeletionToAdministracjaGroupMixin

from django.contrib import admin


# Proste tabele
class Charakter_FormalnyAdmin(
    RestrictDeletionToAdministracjaGroupMixin, BaseBppAdminMixin, DraggableMPTTAdmin
):
    list_display_links = ["nazwa"]
    list_display = [
        "tree_actions",
        "nazwa",
        "skrot",
        "publikacja",
        "streszczenie",
        "nazwa_w_primo",
        "charakter_pbn",
        "charakter_sloty",
        "rodzaj_pbn",
    ]
    list_filter = (
        "publikacja",
        "streszczenie",
        "nazwa_w_primo",
        "charakter_pbn",
        "rodzaj_pbn",
    )
    search_fields = ["skrot", "nazwa"]

    change_list_template = "admin/grappelli_mptt_change_list.html"


admin.site.register(Charakter_Formalny, Charakter_FormalnyAdmin)



# Generated by Django 2.1.7 on 2019-08-04 18:00

from django.db import migrations, models
import django.db.models.deletion

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0169_dyscyplina_change_trigger_cacher'),
    ]

    operations = [
        migrations.CreateModel(
            name='Poziom_Wydawcy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rok', models.IntegerField()),
                ('poziom', models.PositiveSmallIntegerField(choices=[(None, 'nieokreślono'), (1, 'poziom I'), (2, 'poziom II')])),
            ],
        ),
        migrations.CreateModel(
            name='Wydawca',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa', models.CharField(max_length=512, unique=True)),
            ],
            options={
                'ordering': ['nazwa'],
                'abstract': False,
            },
        ),
        migrations.RenameField(
            model_name='praca_doktorska',
            old_name='wydawnictwo',
            new_name='wydawca_opis',
        ),
        migrations.RenameField(
            model_name='praca_habilitacyjna',
            old_name='wydawnictwo',
            new_name='wydawca_opis',
        ),
        migrations.RenameField(
            model_name='wydawnictwo_zwarte',
            old_name='wydawnictwo',
            new_name='wydawca_opis',
        ),
        migrations.AddField(
            model_name='poziom_wydawcy',
            name='wydawca',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Wydawca'),
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='wydawca',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Wydawca'),
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='wydawca',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Wydawca'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='wydawca',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Wydawca'),
        ),
        migrations.AlterUniqueTogether(
            name='poziom_wydawcy',
            unique_together={('rok', 'wydawca')},
        ),
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0170_wydawnictwo_wydawca")
        ),

    ]



from datetime import timedelta

import pytest
from django.urls import reverse
from model_bakery import baker

from django.utils.timezone import localtime

from bpp.models import Praca_Habilitacyjna


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_detail(client, praca_habilitacyjna):
    res = client.get(
        reverse("api_v1:praca_habilitacyjna-detail", args=(praca_habilitacyjna.pk,))
    )
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_list(client, praca_habilitacyjna):
    res = client.get(reverse("api_v1:praca_habilitacyjna-list"))
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_filtering_1(api_client, praca_habilitacyjna):
    czas = localtime(praca_habilitacyjna.ostatnio_zmieniony).strftime(
        "%Y-%m-%d %H:%M:%S"
    )

    res = api_client.get(
        reverse("api_v1:praca_habilitacyjna-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 1


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_filtering_2(api_client, praca_habilitacyjna):
    czas = localtime(
        praca_habilitacyjna.ostatnio_zmieniony + timedelta(seconds=1)
    ).strftime("%Y-%m-%d %H:%M:%S")

    res = api_client.get(
        reverse("api_v1:praca_habilitacyjna-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 0


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_filtering_rok(
    api_client, praca_habilitacyjna, rok
):
    res = api_client.get(
        reverse("api_v1:praca_habilitacyjna-list") + f"?rok_min={rok+1}"
    )
    assert res.json()["count"] == 0


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_ukryj_status(
    api_client, praca_habilitacyjna, uczelnia, przed_korekta, po_korekcie
):

    res = api_client.get(reverse("api_v1:praca_habilitacyjna-list"))
    assert res.json()["count"] == 1

    praca_habilitacyjna.status_korekty = przed_korekta
    praca_habilitacyjna.save()

    uczelnia.ukryj_status_korekty_set.create(status_korekty=przed_korekta)
    res = api_client.get(reverse("api_v1:praca_habilitacyjna-list"))
    assert res.json()["count"] == 0


@pytest.fixture
def wiele_prac_habilitacyjnych(db, typy_odpowiedzialnosci):
    for a in range(100):
        baker.make(Praca_Habilitacyjna)


@pytest.mark.django_db
def test_rest_api_praca_habilitacyjna_no_queries(
    wiele_prac_habilitacyjnych,
    django_assert_max_num_queries,
    api_client,
):
    with django_assert_max_num_queries(11):
        api_client.get(reverse("api_v1:praca_habilitacyjna-list"))



# Generated by Django 2.2.10 on 2020-03-15 19:40

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0203_auto_20200315_2021"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0204_cache_punktacja_autora_query_view"
            )
        )
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models
from django.db.migrations.operations.special import RunSQL


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0047_merge'),
    ]

    operations = [
        RunSQL("UPDATE bpp_jednostka SET nazwa = trim(nazwa)"),
        RunSQL("UPDATE bpp_jednostka SET nazwa = replace(nazwa, '  ', ' ')"),
        RunSQL("UPDATE bpp_jednostka SET nazwa = replace(nazwa, '  ', ' ')"),
        RunSQL("UPDATE bpp_wydzial SET nazwa = 'Jednostki Ogólnouczelniane' where nazwa = 'Jednostka międzywydziałowa'")

    ]



# Generated by Django 3.0.14 on 2022-04-12 21:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0314_punktacja_zrodla_kwarty"),
        ("import_pracownikow", "0005_auto_20210307_1204"),
    ]

    operations = [
        migrations.AddField(
            model_name="importpracownikowrow",
            name="tytul",
            field=models.ForeignKey(
                null=True, on_delete=django.db.models.deletion.SET_NULL, to="bpp.Tytul"
            ),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-03 09:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0153_django21'),
    ]

    operations = [
        migrations.AlterField(
            model_name='patent_autor',
            name='autor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_autor',
            name='autor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte_autor',
            name='autor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Autor'),
        ),
    ]



from rest_framework import serializers

# Serializers define the API representation.
from api_v1.serializers.util import AbsoluteUrlSerializerMixin, AbsoluteUrlField
from bpp.models import Zrodlo, Rodzaj_Zrodla
from bpp.models.nagroda import Nagroda


class NagrodaSerializer(serializers.ModelSerializer):
    object = AbsoluteUrlField(read_only=True)

    organ_przyznajacy = serializers.StringRelatedField()

    class Meta:
        model = Nagroda
        fields = [
            "id",
            "object",
            "nazwa",
            "organ_przyznajacy",
            "rok_przyznania",
            "uzasadnienie",
            "adnotacja",
        ]



# -*- encoding: utf-8 -*-
from optparse import make_option, OptionError
import sys

from django.contrib.sites.models import Site
from django.core.management import BaseCommand

from bpp.models.cache import Rekord


class Command(BaseCommand):
    help = 'Ustawia nazwę domeny i strony internetowej'

    def add_arguments(self, parser):
        parser.add_argument("-d", "--domain", action="store")
        parser.add_argument("-n", "--name", action="store")

    def handle(self, *args, **options):
        kw = {}
        for elem in 'domain', 'name':
            if options[elem]: kw[elem] = options[elem]

        if not kw:
            raise OptionError("Podaj parametr --domain, --name lub oba!", "domain")

        if Site.objects.all().count() != 1:
            raise ValueError("To polecenie jest przeznaczone tylko dla sytuacji, gdy jest jeden obiekt Site w systemie")

        s = Site.objects.all()[0]
        [setattr(s, attr, value) for attr, value in list(kw.items())]
        s.save()


import traceback

from celery.utils.log import get_task_logger
from django.db import transaction
from django.urls import reverse

import notifications
from django_bpp.celery_tasks import app
from import_dyscyplin.models import Import_Dyscyplin
from notifications.models import Notification

logger = get_task_logger("django")


@app.task(bind=True)
def stworz_kolumny(self, pk):
    self.update_state(state="RUNNING")
    i = Import_Dyscyplin.objects.get(pk=pk)

    try:
        with transaction.atomic():
            Import_Dyscyplin.objects.select_for_update().filter(pk=pk)
            try:
                i.stworz_kolumny()
            except Exception:
                logger.exception("Podczas tworzenia kolumn")
                i.info = traceback.format_exc()
            finally:
                i.save(update_fields=["stan", "modified", "info"])
    finally:
        i = Import_Dyscyplin.objects.get(pk=pk)
        self.update_state(state="DONE")
        i.task_id = None
        i.save()

        Notification.objects.send_redirect(
            i, "%s?notification=1" % reverse("import_dyscyplin:detail", args=(i.pk,)),
        )


@app.task(bind=True)
def przeanalizuj_import_dyscyplin(self, pk):
    self.update_state(state="RUNNING")
    i = Import_Dyscyplin.objects.get(pk=pk)

    try:
        with transaction.atomic():
            Import_Dyscyplin.objects.select_for_update().filter(pk=pk)
            try:
                i.przeanalizuj()
                i.integruj_dyscypliny()
                i.sprawdz_czy_poprawne()
                i.sprawdz_czy_konieczne()
            except Exception:
                logger.exception("Podczas analizy pliku")
                i.info = traceback.format_exc()
            finally:
                i.save(update_fields=["stan", "modified", "info"])
    finally:
        # Odśwież obiekt, bo być może web_page_uid się zmieniło
        # i.refresh_from_db() nie zadziała, bo django-fsm zablokuje bezpośrednią
        # modyfukację pola 'status'
        i = Import_Dyscyplin.objects.get(pk=pk)
        self.update_state(state="DONE")

        i.task_id = None
        i.save()

        Notification.objects.send_redirect(
            i, "%s?notification=1" % reverse("import_dyscyplin:detail", args=(i.pk,)),
        )


@app.task(bind=True)
def integruj_import_dyscyplin(self, pk):
    self.update_state(state="RUNNING")
    i = Import_Dyscyplin.objects.get(pk=pk)

    try:
        with transaction.atomic():
            Import_Dyscyplin.objects.select_for_update().filter(pk=pk)
            try:
                i.integruj_wiersze()
            except Exception:
                logger.exception("Podczas integracji wierszy")
                i.info = traceback.format_exc()
            finally:
                i.save(update_fields=["stan", "modified", "info"])

    finally:
        i = Import_Dyscyplin.objects.get(pk=pk)
        self.update_state(state="DONE")
        i.task_id = None
        i.save()

        Notification.objects.send_redirect(
            i, "%s?notification=1" % reverse("import_dyscyplin:detail", args=(i.pk,)),
        )



# Generated by Django 3.0.7 on 2020-08-03 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("notifications", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="notification",
            name="acknowledged",
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.0.7 on 2020-07-27 12:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0214_auto_20200628_2344"),
    ]

    operations = [
        migrations.AddField(
            model_name="praca_doktorska",
            name="numer_odbitki",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="praca_habilitacyjna",
            name="numer_odbitki",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle",
            name="numer_odbitki",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte",
            name="numer_odbitki",
            field=models.CharField(blank=True, max_length=50, null=True),
        ),
        migrations.AlterField(
            model_name="charakter_formalny",
            name="charakter_ogolny",
            field=models.CharField(
                choices=[
                    ("art", "Artykuł"),
                    ("ksi", "Książka"),
                    ("roz", "Rozdział"),
                    ("xxx", "Inne"),
                ],
                default="xxx",
                help_text="Charakter ogólny uzywany jest m.in. do generowania opisów bibliograficznych. Stanowi on\n        ogólne określenie rekordu, czy jest to książka, rozdział czy coś innego. ",
                max_length=3,
            ),
        ),
    ]



# Generated by Django 3.2.14 on 2022-08-01 19:28

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0003_auto_20220801_2045"),
    ]

    operations = [
        migrations.AddField(
            model_name="zgloszenie_publikacji_autor",
            name="rok",
            field=models.PositiveSmallIntegerField(default=2022),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
                validators=[django.core.validators.MinValueValidator(0)],
            ),
        ),
    ]



# def pobierz_skasowane_prace(client: PBNClient):
#     pobierz_mongodb(
#         client.get_publications(status=DELETED, page_size=50),
#         Publication,
#         pbar_label="pobierz_skasowane_prace",
#     )


def _init(_client, _data):
    import django

    django.setup()

    global client, pobierz_mongodb, Publication, zapisz_mongodb, data
    client = _client
    data = _data

    from pbn_api.integrator import pobierz_mongodb as _p

    pobierz_mongodb = _p

    from pbn_api.integrator import zapisz_mongodb as _p

    zapisz_mongodb = _p

    from pbn_api.models import Publication as _p

    Publication = _p


def _single_task(n):
    for elem in data.fetch_page(n):
        zapisz_mongodb(elem, Publication)


def pobierz_skasowane_prace(client):
    """Odświeża wszystkie publikacje, które są w lokalnej tabeli."""

    from multiprocessing import Pool

    from pbn_api.integrator import DELETED

    from bpp.util import pbar

    data = client.get_publications(status=DELETED, page_size=200)

    pool = Pool(processes=12, initializer=_init, initargs=(client, data))

    for _ in pbar(
        pool.imap_unordered(
            _single_task,
            range(0, data.total_pages),
        ),
        count=data.total_pages,
        label="pobierz_skasowane_prace",
    ):
        pass

    pool.close()
    pool.join()



# Generated by Django 2.1.7 on 2019-07-01 11:40

import bpp.models.cache
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0164_auto_20190701_1303'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cache_Punktacja_Autora',
            fields=[
                ('id', bpp.models.cache.TupleField(base_field=models.IntegerField(), primary_key=True, serialize=False, size=2)),
                ('pkdaut', models.DecimalField(decimal_places=4, max_digits=20)),
                ('slot', models.DecimalField(decimal_places=4, max_digits=20)),
                ('autor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Autor')),
                ('dyscyplina', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Dyscyplina_Naukowa')),
            ],
        ),
        migrations.CreateModel(
            name='Cache_Punktacja_Dyscypliny',
            fields=[
                ('id', bpp.models.cache.TupleField(base_field=models.IntegerField(), primary_key=True, serialize=False, size=2)),
                ('pkd', models.DecimalField(decimal_places=4, max_digits=20)),
                ('slot', models.DecimalField(decimal_places=4, max_digits=20)),
                ('dyscyplina', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Dyscyplina_Naukowa')),
            ],
        ),
    ]



# Generated by Django 2.1.7 on 2019-07-02 10:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0165_cache_punktacja_autora_cache_punktacja_dyscypliny'),
    ]

    operations = [
        migrations.RenameField(
            model_name='cache_punktacja_autora',
            old_name='id',
            new_name='rekord_id',
        ),
        migrations.RenameField(
            model_name='cache_punktacja_dyscypliny',
            old_name='id',
            new_name='rekord_id',
        ),
    ]



# -*- encoding: utf-8 -*-

import logging
from argparse import FileType
from pathlib import Path

from django.contrib.auth import get_user_model
from django.core.files.base import File
from django.core.management.base import BaseCommand

from import_dyscyplin.models import Import_Dyscyplin
from import_dyscyplin.tasks import (
    integruj_import_dyscyplin,
    przeanalizuj_import_dyscyplin,
    stworz_kolumny,
)

logger = logging.getLogger("django")


class Command(BaseCommand):
    help = "Importuje dyscypliny z zadanego pliku"

    def add_arguments(self, parser):
        parser.add_argument("rok", type=int)
        parser.add_argument("path", type=FileType("rb"))

    def handle(self, rok, path, verbosity, *args, **options):
        if verbosity > 1:
            logger.setLevel(logging.DEBUG)

        i_d = Import_Dyscyplin.objects.create(
            owner=get_user_model().objects.first(), rok=rok
        )

        i_d.plik.save(name=Path(path.name).name, content=File(path))

        logger.debug("Tworzę kolumny")
        stworz_kolumny.apply(args=(i_d.pk,))

        i_d = Import_Dyscyplin.objects.get(pk=i_d.pk)
        i_d.zatwierdz_kolumny()
        i_d.save()

        logger.debug("Analizuję plik")
        przeanalizuj_import_dyscyplin.apply(args=(i_d.pk,))

        logger.debug(
            "Poprawne wiersze:    %s" % i_d.poprawne_wiersze_do_integracji().count()
        )
        logger.debug("Niepoprawne wiersze: %s" % i_d.niepoprawne_wiersze().count())

        for elem in i_d.niepoprawne_wiersze().order_by("row_no"):
            logger.debug(
                f"{elem.row_no}: {elem.stan} {elem.info} {elem.nazwisko} {elem.imiona}"
            )

        logger.debug("Integruję plik")
        integruj_import_dyscyplin.apply(args=(i_d.pk,))

        i_d.delete()



# Generated by Django 3.0.11 on 2021-03-07 22:29

from django.db import migrations, models

from bpp.migration_util import load_custom_sql


def ustaw_PAW_ZAW_PW(apps, schema_editor):
    Charakter_Formalny = apps.get_model("bpp", "Charakter_Formalny")
    for skrot in ["PAW", "ZAW"]:
        try:
            c = Charakter_Formalny.objects.get(skrot=skrot)
        except Charakter_Formalny.DoesNotExist:
            continue

        c.wliczaj_do_rankingu = False
        c.save()

    Typ_KBN = apps.get_model("bpp", "Typ_KBN")
    try:
        tk = Typ_KBN.objects.get(skrot="PW")
        tk.wliczaj_do_rankingu = False
        tk.save()
    except Typ_KBN.DoesNotExist:
        pass


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0243_auto_20210307_1452"),
    ]

    operations = [
        migrations.AddField(
            model_name="charakter_formalny",
            name="wliczaj_do_rankingu",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="typ_kbn",
            name="wliczaj_do_rankingu",
            field=models.BooleanField(default=True),
        ),
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0244_nowe_sumy_wliczaj_do")
        ),
        migrations.RunPython(ustaw_PAW_ZAW_PW),
    ]



import pytest

from bpp.models.sloty.core import ISlot
from bpp.models.sloty.exceptions import CannotAdapt
from bpp.models.sloty.wydawnictwo_ciagle import (
    SlotKalkulator_Wydawnictwo_Ciagle_Prog2,
    SlotKalkulator_Wydawnictwo_Ciagle_Prog3,
)


@pytest.mark.django_db
def test_referat_zle_punkty(referat_z_dyscyplinami):
    referat_z_dyscyplinami.punkty_kbn = 31337
    with pytest.raises(CannotAdapt):
        ISlot(referat_z_dyscyplinami)


@pytest.mark.django_db
def test_referat_15_pkt_wos(referat_z_dyscyplinami, baza_wos):
    referat_z_dyscyplinami.punkty_kbn = 15
    referat_z_dyscyplinami.rok = 2021
    referat_z_dyscyplinami.zewnetrzna_baza_danych.create(baza=baza_wos)
    ISlot(referat_z_dyscyplinami)


@pytest.mark.django_db
def test_referat_200_pkt(referat_z_dyscyplinami, baza_wos):
    referat_z_dyscyplinami.punkty_kbn = 200
    referat_z_dyscyplinami.rok = 2021
    ISlot(referat_z_dyscyplinami)


@pytest.mark.django_db
def test_referat_70_pkt(referat_z_dyscyplinami, baza_wos):
    referat_z_dyscyplinami.punkty_kbn = 70
    referat_z_dyscyplinami.rok = 2021
    ISlot(referat_z_dyscyplinami)


@pytest.mark.django_db
def test_referat_20_pkt_z_wydawca(referat_z_dyscyplinami, wydawca):
    referat_z_dyscyplinami.punkty_kbn = 20
    referat_z_dyscyplinami.rok = 2020
    referat_z_dyscyplinami.wydawca = wydawca
    referat_z_dyscyplinami.save()

    wydawca.poziom_wydawcy_set.create(rok=2020, poziom=1)
    res = ISlot(referat_z_dyscyplinami)
    assert res.__class__ == SlotKalkulator_Wydawnictwo_Ciagle_Prog2


@pytest.mark.django_db
def test_referat_20_pkt_bez_wydawcy(referat_z_dyscyplinami):
    referat_z_dyscyplinami.punkty_kbn = 20
    referat_z_dyscyplinami.rok = 2020
    res = ISlot(referat_z_dyscyplinami)
    assert res.__class__ == SlotKalkulator_Wydawnictwo_Ciagle_Prog3


@pytest.mark.django_db
def test_referat_5_pkt_bez_wydawcy(referat_z_dyscyplinami):
    rok = 2021
    referat_z_dyscyplinami.punkty_kbn = 5
    referat_z_dyscyplinami.rok = rok
    res = ISlot(referat_z_dyscyplinami)
    assert res.__class__ == SlotKalkulator_Wydawnictwo_Ciagle_Prog3



import pytest
from django.core import mail
from django.urls import reverse


@pytest.mark.django_db
def test_password_reset(webtest_app, normal_django_user):
    EMAIL = "foo@bar.pl"
    normal_django_user.email = EMAIL
    normal_django_user.save()

    url = reverse("password_reset")
    page = webtest_app.get(url)  # noqa
    page.forms[0]["email"] = EMAIL
    page = page.forms[0].submit().maybe_follow()

    assert page.status_code == 200
    assert len(mail.outbox) == 1



"""
This file was generated with the customdashboard management command, it
contains the two classes for the main dashboard and app index dashboard.
You can customize these classes as you want.

To activate your index dashboard add the following to your settings.py::
    ADMIN_TOOLS_INDEX_DASHBOARD = 'django_bpp.dashboard.CustomIndexDashboard'

And to activate the app index dashboard::
    ADMIN_TOOLS_APP_INDEX_DASHBOARD = 'django_bpp.dashboard.CustomAppIndexDashboard'
"""

from admin_tools.dashboard import AppIndexDashboard, Dashboard, modules

from django.utils.translation import ugettext_lazy as _

from bpp.const import GR_WPROWADZANIE_DANYCH


class CustomIndexDashboard(Dashboard):
    def init_with_context(self, context):

        user = context["request"].user

        if user.groups.filter(name="dane systemowe"):
            self.children.append(
                modules.ModelList(
                    "Dane systemowe",
                    [
                        "bpp.models.charakter_formalny",
                        "bpp.models.funkcja_autora",
                        "bpp.models.informacjaz",
                        "bpp.models.jezyk",
                        "bpp.models.rodzaj_autora",
                        "bpp.models.status_korekty",
                        "bpp.models.tytul",
                        "bpp.models.typ_kbn",
                    ],
                )
            )

        if user.groups.filter(name="struktura"):
            self.children.append(
                modules.ModelList(
                    "Struktura",
                    [
                        "bpp.models.uczelnia",
                        "bpp.models.wydzial",
                        "bpp.models.jednostka",
                    ],
                )
            )

        if user.groups.filter(name=GR_WPROWADZANIE_DANYCH):
            self.children.append(
                modules.ModelList(
                    "Wprowadzanie danych",
                    [
                        "bpp.models.autor",
                        "bpp.models.zrodlo",
                        "bpp.models.bibliografia",
                    ],
                )
            )

        if user.groups.filter(name="administracja"):
            self.children.append(
                modules.ModelList(
                    "Administracja",
                    [
                        "auth.contrib.group",
                        "auth.contrib.user",
                        "multiseek.models.searchform",
                    ],
                )
            )
        # append an app list module for "Applications"

        if user.is_superuser:
            self.children.append(
                modules.AppList(
                    _("Django"),
                )
            )

        # append a recent actions module
        self.children.append(modules.RecentActions(_("Recent Actions"), 5))


class CustomAppIndexDashboard(AppIndexDashboard):
    """
    Custom app index dashboard for django_bpp.
    """

    # we disable title because its redundant with the model list module
    title = ""

    def __init__(self, *args, **kwargs):
        AppIndexDashboard.__init__(self, *args, **kwargs)

        # append a model list module and a recent actions module
        self.children += [
            modules.ModelList(self.app_title, self.models),
            modules.RecentActions(
                _("Recent Actions"), include_list=self.get_app_content_types(), limit=5
            ),
        ]

    def init_with_context(self, context):
        """
        Use this method if you need to access the request context.
        """
        return super().init_with_context(context)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-14 17:17
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0123_kod_dyscyplin'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='autor_dyscyplina',
            unique_together=set([('rok', 'autor')]),
        ),
    ]



import pytest

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from model_bakery import baker

from bpp.models.system import Charakter_Formalny, Jezyk
from bpp.models.wydawnictwo_zwarte import Wydawnictwo_Zwarte
from bpp.views.raporty.raport_aut_jed_common import (
    get_base_query_autor,
    raport_autorow_tabela,
)


def test_raport_autorow(app, autor_jan_kowalski, jednostka, standard_data):
    ksp = Charakter_Formalny.objects.get(skrot="KSP")
    pol = Jezyk.objects.get(skrot="pol.")

    praca_2_2 = baker.make(
        Wydawnictwo_Zwarte, punkty_kbn=5, charakter_formalny=ksp, rok=2015
    )
    praca_2_2.dodaj_autora(
        autor_jan_kowalski, jednostka, typ_odpowiedzialnosci_skrot="aut."
    )

    praca_2_6 = baker.make(
        Wydawnictwo_Zwarte, punkty_kbn=5, charakter_formalny=ksp, jezyk=pol, rok=2015
    )
    praca_2_6.dodaj_autora(
        autor_jan_kowalski, jednostka, typ_odpowiedzialnosci_skrot="red."
    )

    page = app.get(reverse("bpp:raport-autorow", args=(autor_jan_kowalski.pk, 2015)))
    assert page.status_code == 200


def test_raport_jednostek(app, autor_jan_kowalski, jednostka, standard_data):
    ksp = Charakter_Formalny.objects.get(skrot="KSP")
    pol = Jezyk.objects.get(skrot="pol.")  # noqa

    praca_2_2 = baker.make(
        Wydawnictwo_Zwarte, punkty_kbn=5, charakter_formalny=ksp, rok=2015
    )
    praca_2_2.dodaj_autora(
        autor_jan_kowalski, jednostka, typ_odpowiedzialnosci_skrot="aut."
    )

    page = app.get(reverse("bpp:raport-jednostek", args=(jednostka.pk, 2015)))
    assert page.status_code == 200


@pytest.mark.django_db
def test_raport_aut_jed_1_4(standard_data, autor_jan_kowalski, jednostka):
    base_query = get_base_query_autor(autor=autor_jan_kowalski, rok_min=0, rok_max=9999)
    raport_autorow_tabela("1_4", base_query, autor_jan_kowalski)


@pytest.mark.django_db
def test_raport_autorow_msword(standard_data, autor_jan_kowalski, webtest_app):
    res = webtest_app.get(
        reverse("bpp:raport-autorow", args=(autor_jan_kowalski.pk, 2016))
        + "?output=msw"
    )
    assert res.content_type == "application/msword"
    assert res.status_code == 200



import os
from unittest.mock import Mock

from integrator2.util import build_mapping, find_header_row, read_xls_data


class MockSheet:
    max_row = 5
    max_column = 5

    def iter_rows(self, min_row, max_row):
        BAD_ROW = [
            Mock(),
        ] * 5
        GOOD_ROW = [Mock(), Mock(), Mock(), Mock(value="test"), Mock()]
        for a in range(3):
            yield BAD_ROW
        yield GOOD_ROW
        for a in range(10):
            yield BAD_ROW


def test_find_header_row():
    assert find_header_row(MockSheet(), "test") == 4


MOCK_WANTED_COLUMNS = {
    "Tytuł/Stopień": "tytul_skrot",
    "NAZWISKO": "B",
    "IMIĘ": "A",
    "NAZWA JEDNOSTKI": "C",
}


def test_build_mapping():
    xls_columns = [
        Mock(value="imię"),
        Mock(value="nazwisko"),
        Mock(value="NAZWA JEDNOSTKI"),
    ]
    wanted_columns = MOCK_WANTED_COLUMNS
    ret = build_mapping(xls_columns, wanted_columns)
    assert ret == ["A", "B", "C"]


def test_build_mapping_alt():
    xls_columns = [
        Mock(value=x)
        for x in [
            "Lp.",
            "TYTUŁ CZASOPISMA",
            "NR ISSN",
            "LICZBA PUNKTÓW ZA PUBLIKACJĘ W CZASOPIŚMIE NAUKOWYM ",
        ]
    ]

    wanted_columns = {
        "TYTUŁ CZASOPISMA": "nazwa",
        "NR ISSN": "issn",
        "NR E-ISSN": "e_issn",
        "LICZBA PUNKTÓW ZA PUBLIKACJĘ W CZASOPIŚMIE NAUKOWYM": "pk",
    }

    ret = build_mapping(xls_columns, wanted_columns)
    assert ret == [None, "nazwa", "issn", "pk"]


def test_read_xls_data():
    gen = read_xls_data(
        os.path.dirname(__file__) + "/xls/lista_a.xlsx",
        {"TYTUŁ CZASOPISMA": "fubar"},
        "Lp.",
    )
    next(gen)
    next(gen)
    res = next(gen)
    assert res["fubar"] == "AAPG BULLETIN"



from __future__ import annotations

import django.test.client
from django.http import HttpResponse
from django.urls import reverse

from bpp.models import Autor
from bpp.tests import normalize_response_content


def test_ImportPracownikowResetujPodstawoweMiejscePracyView_nic_do_odpiecia(
    import_pracownikow_performed,
    uczelnia_z_obca_jednostka,
    admin_client: django.test.client.Client,
):
    url = reverse(
        "import_pracownikow:importpracownikow-results",
        kwargs={"pk": import_pracownikow_performed.pk},
    )
    res: HttpResponse = admin_client.get(url)
    assert "tego pliku nie powoduje" in normalize_response_content(res)


def test_ImportPracownikowResetujPodstawoweMiejscePracyView_czlowiek_do_odpiecia(
    import_pracownikow_performed,
    uczelnia_z_obca_jednostka,
    admin_client: django.test.client.Client,
    autor_spoza_pliku: Autor,
    jednostka_spoza_pliku,
):
    autor_spoza_pliku.dodaj_jednostke(jednostka_spoza_pliku)

    url = reverse(
        "import_pracownikow:importpracownikow-results",
        kwargs={"pk": import_pracownikow_performed.pk},
    )
    res: HttpResponse = admin_client.get(url)
    assert "tego pliku nie powoduje" not in normalize_response_content(res)


def test_ImportPracownikowResetujPodstawoweMiejscePracyView_autor_do_odpiecia(
    import_pracownikow_performed,
    uczelnia_z_obca_jednostka,
    admin_client: django.test.client.Client,
    autor_spoza_pliku,
    jednostka_spoza_pliku,
):
    autor_spoza_pliku.dodaj_jednostke(jednostka_spoza_pliku)
    autor_spoza_pliku.refresh_from_db()
    assert autor_spoza_pliku.aktualna_jednostka == jednostka_spoza_pliku

    url = reverse(
        "import_pracownikow:importpracownikow-resetuj-podstawowe-miejsce-pracy",
        kwargs={"pk": import_pracownikow_performed.pk},
    )
    admin_client.get(url)

    autor_spoza_pliku.refresh_from_db()
    assert autor_spoza_pliku.aktualna_jednostka is None



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-08 19:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0079_wydzial_skrot_nazwy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='wydzial',
            name='skrot',
            field=models.CharField(help_text='Skrót nazwy wydziału, wersja minimalna, np. "WL"', max_length=10, unique=True, verbose_name='Skrót'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-25 20:35

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0156_auto_20190319_1648'),
        ('bpp', '0157_auto_20190324_1834'),
    ]

    operations = [
    ]



# Generated by Django 3.2.14 on 2022-07-08 11:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("import_pracownikow", "0007_django32"),
    ]

    operations = [
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="podstawowe_miejsce_pracy",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 3.0.14 on 2021-08-08 23:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0022_auto_20210809_0120"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="scientist",
            unique_together={
                (
                    "mongoId",
                    "lastName",
                    "name",
                    "pbnId",
                    "qualifications",
                    "orcid",
                    "polonUid",
                )
            },
        ),
    ]



"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""
from datetime import datetime
from django.contrib.auth import get_user_model

from django.test import TestCase
from celeryui.interfaces import IWebTask
from celeryui.models import Report, Status
from celeryui.registry import ReportAdapter, registerAdapter

User = get_user_model()


class SomeAdapter(ReportAdapter):
    def get_readable_title(self):
        return self.original.function.lower().replace("_", " ")

registerAdapter('MAD_EXAMPLE', SomeAdapter)


class TestModels(TestCase):

    def setUp(self):
        u = User.objects.create(username='foo', password='bar')

        r = Report.objects.create(
            ordered_by=u, function='foo', arguments={},
            file=None, started_on=None, finished_on=None,
            progress=0.0)

        self.r = r
        self.u = u

    def test_raport_status(self):
        r = self.r
        self.assertEqual(r.status(), Status.WAITING)

        r.started_on= datetime.now()
        self.assertEqual(r.status(), Status.IN_PROGRESS)

        r.finished_on = datetime.now()
        self.assertEqual(r.status(), Status.COMPLETED)

        r.error = True
        self.assertEqual(r.status(), Status.ERROR)

    def test_adaptation(self):
        r = self.r
        r.function = "MAD_EXAMPLE"
        self.assertEqual(
            IWebTask(self.r).get_readable_title(),
            'mad example')

    def test_context_okay(self):
        def fokay():
            return True

        r = self.r
        self.assertEqual(r.finished_on, None)
        r.run_in_context(fokay)
        self.assertNotEqual(r.finished_on, None)

    def test_context_error(self):
        def ferror():
            1/0

        r = self.r

        self.assertEqual(r.finished_on, None)
        r.run_in_context(ferror)
        self.assertNotEqual(r.finished_on, None)
        self.assertEqual(r.error, True)
        self.assertIn("ZeroDivisionError", r.traceback)

        self.assertRaises(
            ZeroDivisionError, r.run_in_context, ferror, raise_exceptions=True)


from datetime import timedelta

import pytest
from model_bakery import baker

from integrator2.models import ListaMinisterialnaIntegration
from integrator2.tasks import analyze_file, remove_old_integrator_files

from django.utils import timezone


@pytest.mark.django_db(transaction=True)
def test_analyze_file(lmi):
    res = analyze_file(pk=lmi.pk)
    assert res is None


@pytest.mark.django_db
def test_remove_old_integrator_files():
    baker.make(ListaMinisterialnaIntegration)
    f = baker.make(ListaMinisterialnaIntegration)
    f.uploaded_on = timezone.now() - timedelta(days=20)
    f.save()

    assert ListaMinisterialnaIntegration.objects.all().count() == 2

    remove_old_integrator_files()

    assert ListaMinisterialnaIntegration.objects.all().count() == 1



# -*- encoding: utf-8 -*-

import pytest


def test_handler403_permission_denied(client):
    try:
        client.get("/admin/bpp/")
    except TypeError:
        assert False


@pytest.mark.django_db
@pytest.mark.urls("bpp.tests.test_views.urls_test_handlers")
def test_handler500(client):
    with pytest.raises(ZeroDivisionError):
        try:
            client.get("/test_500")
        except TypeError as e:
            assert False


@pytest.mark.django_db
def test_handler404_not_found(client):
    try:
        client.get("/bpp/this_page_does_not_exist_for_sure")
    except TypeError:
        assert False



import pytest
from model_bakery import baker

from ewaluacja2021.models import IloscUdzialowDlaAutora, LiczbaNDlaUczelni

from bpp.models import Autor_Dyscyplina, Wydawnictwo_Ciagle


@pytest.fixture
def genetyczny_3n(
    uczelnia,
    autor_jan_nowak,
    dyscyplina1,
    jednostka,
    denorms,
    typ_odpowiedzialnosci_autor,
):
    """
    Duży test integracyjny, sprawdza generowanie raportu genetycznego
    3N na bazie jednej pracy.
    """
    ROK = 2021

    Autor_Dyscyplina.objects.create(
        autor=autor_jan_nowak, rok=ROK, dyscyplina_naukowa=dyscyplina1
    )

    IloscUdzialowDlaAutora.objects.create(
        autor=autor_jan_nowak,
        ilosc_udzialow=10,
        ilosc_udzialow_monografie=10,
        dyscyplina_naukowa=dyscyplina1,
    )

    LiczbaNDlaUczelni.objects.create(
        dyscyplina_naukowa=dyscyplina1, uczelnia=uczelnia, liczba_n=100
    )

    for a in range(20):

        wc: Wydawnictwo_Ciagle = baker.make(
            Wydawnictwo_Ciagle,
            rok=ROK,
            punkty_kbn=5,
            tytul_oryginalny="Czy z tytułu znikły <i>wszystkie</i> tagi <sup>HTML</sup>?",
        )
        wc.dodaj_autora(autor_jan_nowak, jednostka, dyscyplina_naukowa=dyscyplina1)

    denorms.flush()

    return {"dyscyplina": dyscyplina1}



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-15 20:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0010_auto_20180415_1107'),
    ]

    operations = [
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='nazwa_jednostki',
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='nazwa_wydzialu',
            field=models.CharField(blank=True, max_length=512, null=True),
        ),
    ]



from pbn_api.admin.base import BasePBNAPIAdmin
from pbn_api.admin.filters import (
    OdpowiednikOswiadczeniaInstytucjiAutorWBPPFilter,
    OdpowiednikOswiadczeniaInstytucjiPublikacjaWBPPFilter,
)
from pbn_api.exceptions import StatementDeletionError
from pbn_api.models import OswiadczenieInstytucji

from django.contrib import admin

from bpp.models import Rekord


@admin.register(OswiadczenieInstytucji)
class OswiadczeniaInstytucjiAdmin(BasePBNAPIAdmin):
    autocomplete_fields = ["institutionId", "personId", "publicationId"]

    list_select_related = ["publicationId", "personId", "institutionId"]

    search_fields = [
        "publicationId__title",
        "publicationId__year",
        "publicationId__pk",
        "personId__lastName",
        "personId__name",
    ]

    list_display = [
        "publicationId",
        "odpowiednik_publikacji_w_bpp",
        "personId",
        "odpowiednik_osoby_w_bpp",
        "area",
        "inOrcid",
        "type",
    ]

    readonly_fields = [
        "addedTimestamp",
        "area",
        "inOrcid",
        "institutionId",
        "personId",
        "publicationId",
        "type",
    ]

    list_filter = [
        OdpowiednikOswiadczeniaInstytucjiPublikacjaWBPPFilter,
        OdpowiednikOswiadczeniaInstytucjiAutorWBPPFilter,
        "type",
        "inOrcid",
    ]

    def odpowiednik_publikacji_w_bpp(self, obj: OswiadczenieInstytucji):
        return Rekord.objects.filter(pbn_uid_id=obj.publicationId_id).first()

    def odpowiednik_osoby_w_bpp(self, obj: OswiadczenieInstytucji):
        return obj.personId.rekord_w_bpp

    def has_delete_permission(self, request, *args, **kw):
        return True

    def delete_model(self, request, obj: OswiadczenieInstytucji, pbn_client=None):
        try:
            obj.sprobuj_skasowac_z_pbn(request, pbn_client)
        except StatementDeletionError as e:
            from django.contrib import messages

            messages.error(
                request,
                f"Skasowanie niemożliwe, PBN API zwróciło błąd {e}. Proszę zignorować komunikat "
                f"o pomyślnym usunięciu rekordu, wcisnąc 2x przycisk 'Wstecz' w przeglądarce i "
                f"spróbować jeszcze raz. Oświadczenie nie zostało usunięte ani z PBN, ani z lokalnej bazy.",
            )
            return
        super(OswiadczeniaInstytucjiAdmin, self).delete_model(request, obj)



from decimal import Decimal

from .common import SlotMixin


class SlotKalkulator_Wydawnictwo_Ciagle_Prog1(SlotMixin):
    """
    Artykuł z czasopisma z listy ministerialnej.
    Dla roku 2017, 2018: punkty KBN >= 30
    """

    def punkty_pkd(self, dyscyplina):
        if self.ma_dyscypline(dyscyplina):
            return self.original.punkty_kbn

    def slot_dla_autora_z_dyscypliny(self, dyscyplina):
        azd = len(self.autorzy_z_dyscypliny(dyscyplina))
        if azd == 0:
            return
        return Decimal("1") / azd

    def slot_dla_dyscypliny(self, dyscyplina):
        if self.ma_dyscypline(dyscyplina):
            return Decimal("1")


class SlotKalkulator_Wydawnictwo_Ciagle_Prog2(SlotMixin):
    """
    Artykuł z czasopisma z listy ministerialnej.

    Dla roku 2017-2018: punkty KBN 20 lub 25
    """

    def punkty_pkd(self, dyscyplina):
        if self.ma_dyscypline(dyscyplina):
            pierwiastek = self.pierwiastek_k_przez_m(dyscyplina)
            if pierwiastek is None:
                return None

            if self.liczba_k(dyscyplina) == 0:
                return 0

            return self.original.punkty_kbn * max(pierwiastek, Decimal("0.1"))

    def slot_dla_autora_z_dyscypliny(self, dyscyplina):
        if not self.ma_dyscypline(dyscyplina):
            return

        azd = len(self.autorzy_z_dyscypliny(dyscyplina))
        if azd > 0:
            return self.pierwiastek_k_przez_m(dyscyplina) * 1 / azd

    def slot_dla_dyscypliny(self, dyscyplina):
        if not self.ma_dyscypline(dyscyplina):
            return

        return self.pierwiastek_k_przez_m(dyscyplina)


class SlotKalkulator_Wydawnictwo_Ciagle_Prog3(SlotMixin):
    """
    Artykuł z czasopisma z listy ministerialnej.

    Dla roku 2017-2018: punkty KBN poniżej 20 lub 5
    """

    def punkty_pkd(self, dyscyplina):
        if self.ma_dyscypline(dyscyplina):
            k_przez_m = self.k_przez_m(dyscyplina)
            if k_przez_m is None:
                return
            if self.liczba_k(dyscyplina) == 0:
                return 0
            return self.original.punkty_kbn * max(k_przez_m, Decimal("0.1"))

    def slot_dla_autora_z_dyscypliny(self, dyscyplina):
        if not self.ma_dyscypline(dyscyplina):
            return
        return self.jeden_przez_wszyscy()

    def slot_dla_dyscypliny(self, dyscyplina):
        if not self.ma_dyscypline(dyscyplina):
            return
        return self.jeden_przez_wszyscy() * len(self.autorzy_z_dyscypliny(dyscyplina))



# Generated by Django 3.0.14 on 2021-10-27 21:20

import django.core.validators
import django.db.models.deletion
from django.db import migrations, models

import ewaluacja2021.fields
import ewaluacja2021.validators

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0306_delete_ewaluacja2021liczbandlauczelni"),
        ("ewaluacja2021", "0002_auto_20211026_1137"),
    ]

    operations = [
        migrations.CreateModel(
            name="ImportMaksymalnychSlotow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "plik",
                    models.FileField(
                        upload_to="",
                        validators=[
                            ewaluacja2021.validators.validate_xlsx,
                            ewaluacja2021.validators.xlsx_header_validator(
                                columns=[
                                    "Stopień / Tytuł",
                                    "Nazwisko",
                                    "Imię",
                                    "ORCID",
                                    "Dyscyplina",
                                    "Uwzględniony w ewaluacji",
                                    "maksymalna suma udziałów jednostkowych - wszystkie dyscypliny",
                                    "maksymalna suma udziałów jednostkowych - monografie",
                                ],
                                max_header_row=100,
                            ),
                        ],
                    ),
                ),
                ("przeanalizowany", models.BooleanField(default=False)),
            ],
        ),
        migrations.AlterField(
            model_name="iloscudzialowdlaautora",
            name="ilosc_udzialow",
            field=ewaluacja2021.fields.LiczbaNField(
                decimal_places=4,
                max_digits=9,
                validators=[django.core.validators.MaxValueValidator(4)],
            ),
        ),
        migrations.CreateModel(
            name="WierszImportuMaksymalnychSlotow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("orig_data", django.contrib.postgres.fields.jsonb.JSONField()),
                ("poprawny", models.NullBooleanField(default=None)),
                ("wymagana_integracja", models.NullBooleanField(default=None)),
                ("zintegrowany", models.BooleanField(default=False)),
                (
                    "matched_autor",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="bpp.Autor",
                    ),
                ),
                (
                    "matched_dyscyplina",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="bpp.Dyscyplina_Naukowa",
                    ),
                ),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ewaluacja2021.ImportMaksymalnychSlotow",
                    ),
                ),
            ],
        ),
    ]



from bpp.const import PBN_UID_LEN


def test_PublisherAdmin_get_search_results_tekst(admin_client):
    url = "/admin/pbn_api/publisher/autocomplete/?term=tekt"
    res = admin_client.get(url, follow=True)
    assert res.status_code == 200


def test_PublisherAdmin_get_search_results_pbn_uid(admin_client):
    x = "A" * PBN_UID_LEN

    url = "/admin/pbn_api/publisher/autocomplete/?term=" + x

    res = admin_client.get(url, follow=True)
    assert res.status_code == 200



#!/usr/bin/env python



import os
import sys

def entry_point():
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "django_bpp.settings.local")
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)

if __name__ == "__main__":
    entry_point()



# Generated by Django 3.0.11 on 2021-01-24 23:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0234_rekord_mat_nadrzedne"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="praca_doktorska",
            options={
                "ordering": ("rok", "tytul_oryginalny"),
                "verbose_name": "praca doktorska",
                "verbose_name_plural": "prace doktorskie",
            },
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="calkowita_liczba_autorow",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Jeżeli dodajesz monografię, wpisz\n        tutaj całkowitą liczbę autorów monografii. Ta informacja zostanie\n        użyta w eksporcie danych do PBN. Jeżeli informacja ta nie zostanie\n        uzupełiona, wartość tego pola zostanie obliczona i będzie to ilość\n        wszystkich autorów przypisanych do danej monografii",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_zwarte",
            name="calkowita_liczba_redaktorow",
            field=models.PositiveIntegerField(
                blank=True,
                help_text="Jeżeli dodajesz monografię, wpisz tutaj całkowitą liczbę\n        redaktorów monografii. Ta informacja zostanie użyta w eksporcie\n        danych do PBN. Jeżeli pole to nie zostanie uzupełnione, wartość ta\n        zostanie obliczona i będzie to ilość wszystkich redaktorów\n        przypisanych do danej monografii",
                null=True,
            ),
        ),
    ]



from io import BytesIO

import openpyxl
import pytest
from django.urls import reverse
from django_webtest import DjangoTestApp, DjangoWebtestResponse
from model_bakery import baker

from bpp.models import Wydawnictwo_Ciagle, Wydawnictwo_Zwarte

NAZWA_LINKU_EKSPORTU = "Eksport"


@pytest.mark.parametrize(
    "urlname,klass",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
    ],
)
def test_xlsx_export_overflow(urlname, klass, admin_app: DjangoTestApp, settings):
    max_allowed_export_items = 5

    settings.BPP_MAX_ALLOWED_EXPORT_ITEMS = max_allowed_export_items

    baker.make(
        klass,
        _quantity=max_allowed_export_items,
        _bulk_create=True,
    )

    page: DjangoWebtestResponse = admin_app.get(
        reverse(f"admin:bpp_{urlname}_changelist")
    )

    with pytest.raises(IndexError):
        page.click(NAZWA_LINKU_EKSPORTU)

    klass.objects.all().first().delete()

    page: DjangoWebtestResponse = admin_app.get(
        reverse(f"admin:bpp_{urlname}_changelist")
    )
    page.click(NAZWA_LINKU_EKSPORTU)


@pytest.mark.parametrize(
    "urlname,klass",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
    ],
)
def test_xlsx_export_data(urlname, klass, admin_app: DjangoTestApp):
    baker.make(klass)

    page: DjangoWebtestResponse = admin_app.get(
        reverse(f"admin:bpp_{urlname}_changelist")
    )

    xlsx_binary_data = page.click(NAZWA_LINKU_EKSPORTU)
    wb = openpyxl.load_workbook(BytesIO(xlsx_binary_data.content))
    assert wb.active["A1"].value == "id"


def test_xlsx_export_nazwy_zamiast_numerkow(
    wydawnictwo_ciagle, admin_app: DjangoTestApp
):
    # fikstura wydawnictwo_ciagle tworzy wydawnictwo cialge z jezykiem polskim

    page: DjangoWebtestResponse = admin_app.get(
        reverse("admin:bpp_wydawnictwo_ciagle_changelist")
    )

    xlsx_binary_data = page.click(NAZWA_LINKU_EKSPORTU)
    wb = openpyxl.load_workbook(BytesIO(xlsx_binary_data.content))
    ws = wb.active

    column_name = "jezyk"
    for column_cell in ws.iter_cols(1, ws.max_column):  # iterate column cell
        if column_cell[0].value == column_name:  # check for your column
            for data in column_cell[1:2]:  # iterate your column
                assert data.value == "pol."



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-05 09:53
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models

from bpp import initial


def wczytaj_rodzaje_praw(apps, schema_editor):
    Rodzaj_Prawa_Patentowego = apps.get_model("bpp", "Rodzaj_Prawa_Patentowego")
    for nazwa in initial.rodzaj_prawa_patentowego:
        Rodzaj_Prawa_Patentowego.objects.create(
            nazwa=nazwa)


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0118_auto_20180221_2335'),
    ]

    operations = [
        migrations.CreateModel(
            name='Rodzaj_Prawa_Patentowego',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa', models.CharField(max_length=512, unique=True)),
            ],
            options={
                'verbose_name': 'rodzaj prawa patentowego',
                'verbose_name_plural': 'rodzaje praw patentowych',
                'ordering': ['nazwa'],
            },
        ),
        migrations.RenameField(
            model_name='patent',
            old_name='z_dnia',
            new_name='data_decyzji',
        ),
        migrations.RemoveField(
            model_name='patent',
            name='numer',
        ),
        migrations.AddField(
            model_name='patent',
            name='data_zgloszenia',
            field=models.DateField(blank=True, null=True, verbose_name='Data zgłoszenia'),
        ),
        migrations.AddField(
            model_name='patent',
            name='numer_prawa_wylacznego',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Numer prawa wyłącznego'),
        ),
        migrations.AddField(
            model_name='patent',
            name='numer_zgloszenia',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Numer zgłoszenia'),
        ),
        migrations.AddField(
            model_name='patent',
            name='wdrozenie',
            field=models.NullBooleanField(verbose_name='Wdrożenie'),
        ),
        migrations.AddField(
            model_name='patent',
            name='wydzial',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    to='bpp.Wydzial'),
        ),
        migrations.AddField(
            model_name='patent',
            name='rodzaj_prawa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    to='bpp.Rodzaj_Prawa_Patentowego'),
        ),
        migrations.RunPython(
            wczytaj_rodzaje_praw,
            migrations.RunPython.noop
        )
    ]



import pytest
from django.urls import reverse
from model_bakery import baker

from raport_slotow.models.uczelnia import RaportSlotowUczelnia


@pytest.fixture
def raport_slotow_uczelnia(uczelnia, admin_user) -> RaportSlotowUczelnia:
    return baker.make(RaportSlotowUczelnia, owner=admin_user)


@pytest.fixture
def raport_slotow_uczelnia_page(
    raport_slotow_uczelnia: RaportSlotowUczelnia,
    admin_app,
    admin_user,
    dyscyplina1,
    praca_z_dyscyplina,
):

    res = admin_app.get(
        reverse(
            "raport_slotow:raportslotowuczelnia-results",
            kwargs={"pk": raport_slotow_uczelnia.id},
        )
    )
    return res


def test_raport_uczelnia_filtry_autor__nazwisko(
    raport_slotow_uczelnia_page,
):
    raport_slotow_uczelnia_page.forms[0]["autor__nazwisko"] = "test"
    assert raport_slotow_uczelnia_page.forms[0].submit().status_code == 200


def test_raport_uczelnia_filtry_dyscyplina(
    raport_slotow_uczelnia_page,
    dyscyplina1,
):

    raport_slotow_uczelnia_page.forms[0]["dyscyplina"] = dyscyplina1.pk
    assert raport_slotow_uczelnia_page.forms[0].submit().status_code == 200


def test_raport_uczelnia_filtry_suma__min(
    raport_slotow_uczelnia_page,
):

    raport_slotow_uczelnia_page.forms[0]["suma__min"] = 5
    assert raport_slotow_uczelnia_page.forms[0].submit().status_code == 200


def test_raport_uczelnia_filtry_slot__min(
    raport_slotow_uczelnia_page,
):

    raport_slotow_uczelnia_page.forms[0]["slot__min"] = 5
    assert raport_slotow_uczelnia_page.forms[0].submit().status_code == 200


def test_raport_uczelnia_filtry_avg__min(
    raport_slotow_uczelnia_page,
):

    raport_slotow_uczelnia_page.forms[0]["avg__min"] = 5
    assert raport_slotow_uczelnia_page.forms[0].submit().status_code == 200



# Generated by Django 3.0.11 on 2021-03-07 10:10

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0242_auto_20210307_1110"),
        ("import_pracownikow", "0003_auto_20210228_1916"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="importpracownikowrow",
            name="integracja_mozliwa",
        ),
        migrations.RemoveField(
            model_name="importpracownikowrow",
            name="wiersz_xls",
        ),
        migrations.AddField(
            model_name="importpracownikow",
            name="integrated",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="importpracownikow",
            name="performed",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="importpracownikowrow",
            name="dane_znormalizowane",
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="importpracownikowrow",
            name="funkcja_autora",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Funkcja_Autora",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="importpracownikowrow",
            name="grupa_pracownicza",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Grupa_Pracownicza",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="importpracownikowrow",
            name="podstawowe_miejsce_pracy",
            field=models.NullBooleanField(),
        ),
        migrations.AddField(
            model_name="importpracownikowrow",
            name="wymiar_etatu",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="bpp.Wymiar_Etatu",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="importpracownikowrow",
            name="zmiany_potrzebne",
            field=models.BooleanField(default=None),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="importpracownikowrow",
            name="parent",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                to="import_pracownikow.ImportPracownikow",
            ),
        ),
    ]



# Generated by Django 3.2.15 on 2022-08-16 10:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0007_auto_20220816_1019"),
    ]

    operations = [
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="ostatnio_zmieniony",
            field=models.DateTimeField(auto_now=True, db_index=True, null=True),
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="przyczyna_zwrotu",
            field=models.TextField(blank=True, null=True),
        ),
    ]



from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Fieldset, Layout
from dal import autocomplete
from django import forms
from django.core.exceptions import ValidationError
from django.forms import inlineformset_factory
from django.forms.widgets import HiddenInput

from zglos_publikacje.models import Zgloszenie_Publikacji, Zgloszenie_Publikacji_Autor
from zglos_publikacje.validators import validate_file_extension_pdf

from bpp.models import Autor, Dyscyplina_Naukowa, Jednostka


class Zgloszenie_Publikacji_DaneOgolneForm(forms.ModelForm):
    rok = forms.IntegerField(
        help_text="Rok publikacji zgłaszanej pracy. Rok na późniejszych etapach używany jest "
        "do ustalenia i zweryfikowania dyscyplin naukowych zgłoszonych przez autorów. "
    )

    tytul_oryginalny = forms.CharField(
        widget=forms.Textarea(attrs={"rows": 2, "cols": 80}),
        help_text="Tytuł pracy. Prosimy o wpisanie samego tytułu. Proszę nie wpisywać źródła, miejsca publikacji, "
        "autorów itp. -- wyłacznie tytuł pracy. ",
    )

    email = forms.EmailField(
        help_text="Prosimy o podanie poprawnego adresu e-mail. W razie problemów ze zgłoszeniem na ten adres "
        "zostanie skierowana dalsza korespondencja."
    )

    class Meta:
        model = Zgloszenie_Publikacji
        fields = [
            "tytul_oryginalny",
            "rodzaj_zglaszanej_publikacji",
            "rok",
            "strona_www",
            "email",
        ]

    def __init__(self, *args, **kw):
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Informacje o publikacji",
                "tytul_oryginalny",
                "rok",
                "strona_www",
                "email",
            ),
        )
        super().__init__(*args, **kw)


class Zgloszenie_Publikacji_Plik(forms.ModelForm):
    plik = forms.FileField(
        required=False,
        help_text="""Ponieważ w poprzednim formularzu nie podano adresu WWW
    ani adresu DOI publikacji, prosimy o załączenie pełnego tekstu pracy w formacie PDF.
    Dodawany plik wyłącznie na potrzeby zarejestrowania rekordu w bazie publikacji -
    do wglądu Biblioteki; nie będzie dalej udostępniany.
""",
        validators=[validate_file_extension_pdf],
    )

    class Meta:
        model = Zgloszenie_Publikacji
        fields = ["plik"]

    def __init__(self, *args, **kw):
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Plik załącznika",
                "plik",
            ),
        )
        super().__init__(*args, **kw)

    def clean_plik(self):
        if self.cleaned_data["plik"] is None:
            raise ValidationError(
                "Kliknij przycisk 'Przeglądaj' aby uzupełnić plik PDF z pełnym tekstem "
                "zgłaszanej publikacji. "
            )


class Zgloszenie_Publikacji_AutorForm(forms.ModelForm):
    class Media:
        js = ["/static/bpp/js/autorform_dependant.js"]

    class Meta:
        model = Zgloszenie_Publikacji_Autor
        fields = ["autor", "jednostka", "dyscyplina_naukowa", "rok"]

    autor = forms.ModelChoiceField(
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:public-autor-autocomplete"),
    )

    jednostka = forms.ModelChoiceField(
        queryset=Jednostka.objects.publiczne(),
        widget=autocomplete.ModelSelect2(url="bpp:public-jednostka-autocomplete"),
    )

    dyscyplina_naukowa = forms.ModelChoiceField(
        queryset=Dyscyplina_Naukowa.objects.all(),
        widget=autocomplete.ModelSelect2(
            url="bpp:dyscyplina-naukowa-przypisanie-autocomplete",
            forward=["autor", "rok"],
        ),
        required=False,
    )

    rok = forms.IntegerField(widget=HiddenInput)

    def __init__(self, *args, **kw):
        self.helper = FormHelper()
        self.helper.form_tag = False
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Autor",
                "autor",
                "jednostka",
                "dyscyplina_naukowa",
                "rok",  # "delete"
            )
        )
        super().__init__(*args, **kw)


Zgloszenie_Publikacji_AutorFormSet = inlineformset_factory(
    Zgloszenie_Publikacji,
    Zgloszenie_Publikacji_Autor,
    form=Zgloszenie_Publikacji_AutorForm,
    extra=1,
    # min_num=1,
    validate_min=True,
)


class Zgloszenie_Publikacji_KosztPublikacjiForm(forms.ModelForm):
    class Meta:
        model = Zgloszenie_Publikacji
        fields = [
            "opl_pub_cost_free",
            "opl_pub_research_potential",
            "opl_pub_research_or_development_projects",
            "opl_pub_other",
            "opl_pub_amount",
        ]

    def __init__(self, *args, **kw):
        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Opłata za publikację",
                "opl_pub_cost_free",
                "opl_pub_research_potential",
                "opl_pub_research_or_development_projects",
                "opl_pub_other",
                "opl_pub_amount",
            ),
        )
        super().__init__(*args, **kw)

    def clean_opl_pub_cost_free(self):
        v = self.cleaned_data.get("opl_pub_cost_free")
        if v is None:
            raise forms.ValidationError("Wybierz jakąś wartość")
        return v



import pytest
from django.urls import reverse
from model_bakery import baker

from pbn_api.models import Publication, SentData

from bpp import const
from bpp.models import Uczelnia, Wydawnictwo_Ciagle
from bpp.tests import normalize_html


def test_wydawnictwo_ciagle_admin_zapisz_bez_linkow(
    admin_app, uczelnia, wydawnictwo_ciagle, charaktery_formalne
):
    url = "admin:bpp_wydawnictwo_ciagle_change"
    page = admin_app.get(reverse(url, args=(wydawnictwo_ciagle.pk,)))

    page.forms["wydawnictwo_ciagle_form"][
        "tytul_oryginalny"
    ].value = "Test www.onet.pl formularza"
    page.forms["wydawnictwo_ciagle_form"].submit().maybe_follow()

    wydawnictwo_ciagle.refresh_from_db()
    assert "a href" not in wydawnictwo_ciagle.tytul_oryginalny


def test_wydawnictwo_ciagle_admin_zapisz_i_wyslij_do_pbn_add_tak(
    admin_app, uczelnia, mocker
):

    uczelnia.pbn_aktualizuj_na_biezaco = True
    uczelnia.pbn_integracja = True
    uczelnia.pbn_client = mocker.Mock()
    uczelnia.save()

    url = "admin:bpp_wydawnictwo_ciagle_add"
    page = admin_app.get(reverse(url))
    assert "Zapisz i wyślij do PBN" in normalize_html(page.content.decode("utf-8"))


def test_wydawnictwo_ciagle_admin_zapisz_i_wyslij_do_pbn_add_nie(
    admin_app, uczelnia, mocker
):

    uczelnia.pbn_aktualizuj_na_biezaco = False
    uczelnia.pbn_integracja = True
    uczelnia.pbn_client = mocker.Mock()
    uczelnia.save()

    url = "admin:bpp_wydawnictwo_ciagle_add"
    page = admin_app.get(reverse(url))
    assert "Zapisz i wyślij do PBN" not in normalize_html(page.content.decode("utf-8"))


def test_wydawnictwo_ciagle_admin_zapisz_i_wyslij_do_pbn_change_tak(
    admin_app, uczelnia, mocker, wydawnictwo_ciagle, charaktery_formalne
):
    orig_pbn_client = Uczelnia.pbn_client

    try:
        pbn_client = mocker.Mock()
        Uczelnia.pbn_client = pbn_client

        uczelnia.pbn_aktualizuj_na_biezaco = True
        uczelnia.pbn_integracja = True
        uczelnia.save()

        cf = wydawnictwo_ciagle.charakter_formalny
        cf.rodzaj_pbn = const.RODZAJ_PBN_ARTYKUL
        cf.save()

        pub = baker.make(Publication)
        SentData.objects.create(object=wydawnictwo_ciagle, data_sent={}, pbn_uid=pub)

        url = "admin:bpp_wydawnictwo_ciagle_change"
        page = admin_app.get(reverse(url, args=(wydawnictwo_ciagle.pk,)))
        assert "Zapisz i wyślij do PBN" in normalize_html(page.content.decode("utf-8"))

        page = (
            page.forms["wydawnictwo_ciagle_form"]
            .submit("_continue_and_pbn")
            .maybe_follow()
        )
        content = normalize_html(page.content.decode("utf-8"))
        assert "pomyślnie zmieniony" in content
        assert len(pbn_client.mock_calls) == 4
    finally:
        Uczelnia.pbn_client = orig_pbn_client


def test_wydawnictwo_ciagle_admin_zapisz_i_wyslij_do_pbn_change_nie(
    admin_app, uczelnia, mocker, wydawnictwo_ciagle
):
    orig_pbn_client = Uczelnia.pbn_client

    try:
        pbn_client = mocker.Mock()

        Uczelnia.pbn_client = pbn_client

        uczelnia.pbn_aktualizuj_na_biezaco = False
        uczelnia.pbn_integracja = True
        uczelnia.save()

        url = "admin:bpp_wydawnictwo_ciagle_change"
        page = admin_app.get(reverse(url, args=(wydawnictwo_ciagle.pk,)))
        assert "Zapisz i wyślij do PBN" not in normalize_html(
            page.content.decode("utf-8")
        )
    finally:
        Uczelnia.pbn_client = orig_pbn_client


TEST_PBN_ID = 50000


@pytest.mark.parametrize(
    "fld,value",
    [
        ("pbn_uid", TEST_PBN_ID),
        ("doi", "10.10/123123"),
        ("www", "https://foobar.pl"),
        ("public_www", "https://foobar.pl"),
    ],
)
def test_Wydawnictwo_Ciagle_Admin_sprawdz_duplikaty_www_doi_pbn(
    admin_app, zrodlo, fld, value
):
    if fld == "pbn_uid":
        value = baker.make(Publication, pk=TEST_PBN_ID)

    baker.make(Wydawnictwo_Ciagle, **{fld: value})
    w2 = baker.make(Wydawnictwo_Ciagle, zrodlo=zrodlo)

    if fld == "pbn_uid":
        value = TEST_PBN_ID  # baker.make(Publication, pk=TEST_PBN_ID)

    url = "admin:bpp_wydawnictwo_ciagle_change"
    page = admin_app.get(reverse(url, args=(w2.pk,)))

    if fld == "pbn_uid":
        page.forms["wydawnictwo_ciagle_form"][fld].force_value(value)
    else:
        page.forms["wydawnictwo_ciagle_form"][fld].value = value
    res = page.forms["wydawnictwo_ciagle_form"].submit().maybe_follow()

    assert "inne rekordy z identycznym polem" in normalize_html(
        res.content.decode("utf-8")
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-20 01:21
from __future__ import unicode_literals

from django.db import migrations

def dodaj_nagrody(apps, schema_editor):
    for skrot, nazwa in [
        ("AW01", "Prezes Rady Ministrów"),
        ("AW02", "minister kierujący działami administracji rządowej"),
        ("AW03", "wydział PAN"),
        ("AW04", "komitet naukowy PAN"),
        ("AW05", "Fundacja na rzecz Nauki Polskiej"),
        ("AW06", "zagraniczne towarzystwo naukowe"),
        ("AW07", "organizacja międzynarodowa lub ogólnopolskie towarzystwo "
                 "naukowe o szczególnym prestiżu")
    ]:
        apps.get_model("bpp", "OrganPrzyznajacyNagrody").objects.create(
            skrot=skrot, nazwa=nazwa)


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0099_nagrody'),
    ]

    operations = [
        migrations.RunPython(
            dodaj_nagrody,
            migrations.RunPython.noop),
    ]



from braces.views import GroupRequiredMixin
from django import forms
from django.db.models import F
from django.views.generic import ListView

from rozbieznosci_if.models import IgnorujRozbieznoscIf

from django.contrib.contenttypes.models import ContentType

from bpp.models import Wydawnictwo_Ciagle


class SetForm(forms.Form):
    _set = forms.IntegerField(min_value=0)


class IgnoreForm(forms.Form):
    _ignore = forms.IntegerField(min_value=0)


class RozbieznosciView(GroupRequiredMixin, ListView):
    template_name = "rozbieznosci_if/index.html"
    group_required = "wprowadzanie danych"
    paginate_by = 150

    def get_queryset(self):
        return (
            Wydawnictwo_Ciagle.objects.exclude(zrodlo=None)
            .filter(
                zrodlo__punktacja_zrodla__rok=F("rok"),
            )
            .exclude(zrodlo__punktacja_zrodla__impact_factor=F("impact_factor"))
            .exclude(
                pk__in=IgnorujRozbieznoscIf.objects.filter(
                    content_type=ContentType.objects.get_for_model(Wydawnictwo_Ciagle),
                ).values_list("object_id")
            )
            .order_by(
                "-ostatnio_zmieniony",
            )
            .select_related("zrodlo")
            .annotate(
                punktacja_zrodla_impact_factor=F(
                    "zrodlo__punktacja_zrodla__impact_factor"
                )
            )
        )

    def get(self, request, *args, **kw):
        if "_ignore" in request.GET:
            frm = IgnoreForm(request.GET)
            if frm.is_valid():
                IgnorujRozbieznoscIf.objects.get_or_create(
                    object_id=frm.cleaned_data["_ignore"],
                    content_type=ContentType.objects.get_for_model(Wydawnictwo_Ciagle),
                )
        if "_set" in request.GET:
            frm = SetForm(request.GET)
            if frm.is_valid():
                IgnorujRozbieznoscIf.objects.filter(
                    object_id=frm.cleaned_data["_set"],
                    content_type=ContentType.objects.get_for_model(Wydawnictwo_Ciagle),
                ).delete()

                try:
                    wc = Wydawnictwo_Ciagle.objects.get(pk=frm.cleaned_data["_set"])
                    if wc.impact_factor != wc.punktacja_zrodla().impact_factor:
                        wc.impact_factor = wc.punktacja_zrodla().impact_factor
                        wc.save()
                except Wydawnictwo_Ciagle.DoesNotExist:
                    pass

        return super(RozbieznosciView, self).get(request, *args, **kw)



import pytest
from model_bakery import baker

from notifications.models import Notification

from bpp.models import Wydawnictwo_Zwarte


@pytest.mark.django_db
def test_Notifications_objects():
    o = baker.make(Wydawnictwo_Zwarte)
    Notification.objects.send_redirect(o, "http://onet.pl")



from datetime import date, datetime, timedelta

import pytest
from django.core.exceptions import ValidationError
from django.db.utils import IntegrityError, InternalError
from model_bakery import baker

from bpp.models.struktura import Jednostka, Jednostka_Wydzial, Uczelnia, Wydzial


@pytest.mark.django_db
def test_jednostka_wydzial_aktualna():
    """Sprawdź, czy dodanie obiektów Jednostka_Wydzial spowoduje zmodyfikowanie
    atrybutów "aktualna" oraz "wydzial_id" na obiekcie Jednostka"""

    u = baker.make(Uczelnia)
    w = baker.make(Wydzial, uczelnia=u)
    j = baker.make(Jednostka, uczelnia=u)

    assert j.wydzial is None

    jw = Jednostka_Wydzial.objects.create(jednostka=j, wydzial=w)

    j.refresh_from_db()
    assert j.wydzial == w
    assert j.aktualna

    jw.do = datetime.now().date() - timedelta(days=30)
    jw.save()

    j.refresh_from_db()
    assert j.wydzial == w
    assert j.aktualna is False

    jw.delete()
    j.refresh_from_db()
    assert j.wydzial is None
    assert j.aktualna is False


@pytest.mark.django_db(transaction=True)
def test_jednostka_before_insert():
    """Sprawdź, że nie da się przypisać jednostki do wydziału z innej uczelni
    przez edycje tabeli bpp_jednostka"""

    u1 = baker.make(Uczelnia)
    u2 = baker.make(Uczelnia)

    w1 = baker.make(Wydzial, uczelnia=u1)
    w2 = baker.make(Wydzial, uczelnia=u2)

    with pytest.raises(InternalError):
        j = baker.make(Jednostka, uczelnia=u2, wydzial=w1)

    j = baker.make(Jednostka, uczelnia=u2, wydzial=w2)

    for elem in [j, w2, w1, u2, u1]:
        elem.delete()


@pytest.mark.django_db(transaction=True)
def test_jednostka_wydzial_before_insert():
    """Sprawdź, że nie da się przypisać jednostki do wydziału z innej uczelni
    przez edycje tabeli bpp_jednostka_wydzial"""

    u1 = baker.make(Uczelnia)
    u2 = baker.make(Uczelnia)

    w1 = baker.make(Wydzial, uczelnia=u1)
    w2 = baker.make(Wydzial, uczelnia=u2)

    j1 = baker.make(Jednostka, uczelnia=u1)
    assert j1.wydzial is None

    jw = Jednostka_Wydzial(jednostka=j1, wydzial=w2)
    with pytest.raises(InternalError):
        jw.save()

    jw = Jednostka_Wydzial.objects.create(jednostka=j1, wydzial=w1)

    for elem in (jw, j1, w2, w1, u2, u1):
        elem.delete()


@pytest.mark.django_db(transaction=True)
def test_jednostka_wydzial_time_trigger():
    """Sprawdź, czy nie da się przypisać jednej tej samej jednostki do dwóch
    wydziałów w tym samym czasie"""

    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)

    w2 = baker.make(Wydzial, uczelnia=u1)
    j1 = baker.make(Jednostka, uczelnia=u1)

    jw1 = Jednostka_Wydzial.objects.create(
        jednostka=j1,
        wydzial=w1,
    )

    jw2 = Jednostka_Wydzial(jednostka=j1, wydzial=w2, od=date(2001, 1, 1))
    with pytest.raises(IntegrityError):
        jw2.save()

    jw2 = Jednostka_Wydzial(jednostka=j1, wydzial=w2, do=date(2001, 1, 1))
    with pytest.raises(IntegrityError):
        jw2.save()

    for elem in u1, w1, j1, jw1:
        elem.delete()


@pytest.mark.django_db(transaction=True)
def test_jednostka_wydzial_bez_dat_do_w_przyszlosci_constraint():
    """Sprawdź, czy przypisanie daty 'do' w przyszłości da błąd"""
    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)
    j1 = baker.make(Jednostka, wydzial=w1, uczelnia=u1)
    jw = Jednostka_Wydzial(
        jednostka=j1, wydzial=w1, od=date.today() - timedelta(days=30), do=date.today()
    )

    with pytest.raises(IntegrityError):
        jw.save()

    for elem in u1, w1, j1:
        elem.delete()


@pytest.mark.django_db(transaction=True)
def test_jednostka_wydzial_time_trigger_delete_1():
    """Sprawdź, czy po zmianie jednostka_id trigger zwróci błąd."""

    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)

    j1 = baker.make(Jednostka, uczelnia=u1)
    j2 = baker.make(Jednostka, uczelnia=u1)

    jw1 = Jednostka_Wydzial.objects.create(
        jednostka=j1,
        wydzial=w1,
    )

    jw1.jednostka = j2

    with pytest.raises(InternalError):
        jw1.save()

    for elem in u1, w1, j1, j2, jw1:
        elem.delete()


@pytest.mark.django_db
def test_jednostka_wydzial_time_trigger_delete_2():
    """Sprawdź, czy po dodaniu wartości do tabeli bpp_jednostka_wydzial
    możliwe będzie usunięcie tychże (że nie wywali wówczas triggera)"""

    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)
    j1 = baker.make(Jednostka, uczelnia=u1)

    jw1 = Jednostka_Wydzial.objects.create(
        jednostka=j1,
        wydzial=w1,
    )

    jw1.delete()

    j1.refresh_from_db()

    assert j1.wydzial is None
    assert j1.aktualna is False

    for elem in u1, w1, j1:
        elem.delete()


@pytest.mark.django_db
def test_jednostka_wydzial_save_trigger_zakres_dat():
    """Sprawdź, walidacja obiektu Jednostka_Wydzial zwróci prawidłowy błąd dla dwóch zachodzących na siebie
    zakresów dat przy przypisaniu."""

    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)
    w2 = baker.make(Wydzial, uczelnia=u1)

    j1 = baker.make(Jednostka, uczelnia=u1)

    jw1 = Jednostka_Wydzial.objects.create(  # noqa
        jednostka=j1, wydzial=w1, od=date(2000, 1, 1), do=date(2000, 2, 1)
    )

    jw2 = Jednostka_Wydzial(
        jednostka=j1, wydzial=w2, od=date(2000, 1, 15), do=date(2000, 2, 20)
    )

    with pytest.raises(ValidationError):
        jw2.clean()

    jw2 = Jednostka_Wydzial(jednostka=j1, wydzial=w2, od=date(2001, 1, 15), do=None)

    # ValidationError nie został podniesiony
    jw2.clean()


@pytest.mark.django_db
def test_jednostka_wydzial_save_trigger_zmiana_jednostka_id():
    """Sprawdź, czy walidacja obiektu Jednostka_Wydzial zwróci prawidłowy błąd przy próbie zmiany jednostka_id,
    któro to z kolei nie jest obsługiwane przez triggery bazodanowe."""

    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)
    w2 = baker.make(Wydzial, uczelnia=u1)  # noqa

    j1 = baker.make(Jednostka, uczelnia=u1)
    j2 = baker.make(Jednostka, uczelnia=u1)

    jw1 = Jednostka_Wydzial.objects.create(
        jednostka=j1, wydzial=w1, od=date(2000, 1, 1), do=date(2000, 2, 1)
    )

    jw1.jednostka = j2

    with pytest.raises(ValidationError):
        jw1.clean()


@pytest.mark.django_db
def test_jednostka_save_trigger_rozne_uczelnie():
    """Sprawdź, czy nie da się przypisać jednej tej samej jednostki do dwóch
    różnych uczelni"""

    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)  # noqa

    u2 = baker.make(Uczelnia)
    w2 = baker.make(Wydzial, uczelnia=u2)

    j1 = baker.make(Jednostka, uczelnia=u1)

    jw = Jednostka_Wydzial(jednostka=j1, wydzial=w2)

    with pytest.raises(ValidationError):
        jw.clean()


@pytest.mark.django_db
def test_jednostka_save_trigger_data_w_przyszlosci():
    """Sprawdź, czy podanie daty "do" w przyszłości nie przejdzie"""
    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)
    j1 = baker.make(Jednostka, wydzial=w1, uczelnia=u1)
    jw = Jednostka_Wydzial(
        jednostka=j1, wydzial=w1, od=date.today() - timedelta(days=30), do=date.today()
    )

    with pytest.raises(ValidationError):
        jw.clean()


@pytest.mark.django_db
def test_jednostka_save_trigger_dwa_zakresy_bug():
    u1 = baker.make(Uczelnia)
    w1 = baker.make(Wydzial, uczelnia=u1)
    w2 = baker.make(Wydzial, uczelnia=u1)
    j1 = baker.make(Jednostka, uczelnia=u1)

    Jednostka_Wydzial.objects.create(
        jednostka=j1, wydzial=w1, od=None, do=datetime(2013, 1, 1)
    )

    jw = Jednostka_Wydzial.objects.create(
        jednostka=j1, wydzial=w2, od=datetime(2013, 1, 2), do=None
    )

    # Jednostka ma mieć ustalony wydział nr 2
    j1.refresh_from_db()
    assert j1.wydzial == w2

    jw.do = datetime(2013, 1, 3)
    jw.save()

    Jednostka_Wydzial.objects.create(
        jednostka=j1, wydzial=w1, od=datetime(2013, 1, 4), do=None
    )

    # Jednostka ma mieć ustalony wydział nr 1
    j1.refresh_from_db()
    assert j1.wydzial == w1


@pytest.mark.django_db
def test_wyczysc_przypisania_wariant_1(wydzial, jednostka):
    Jednostka_Wydzial.objects.create(
        wydzial=wydzial, jednostka=jednostka, od=None, do=date(2012, 6, 1)
    )
    Jednostka_Wydzial.objects.wyczysc_przypisania(
        jednostka, date(2012, 1, 1), date(2012, 12, 31)
    )
    assert jednostka.wydzial_dnia(date(2011, 12, 31)) == wydzial
    assert jednostka.wydzial_dnia(date(2012, 1, 1)) is None
    assert jednostka.wydzial_dnia(date(2012, 6, 1)) is None
    assert jednostka.wydzial_dnia(date(2012, 12, 31)) is None
    assert jednostka.wydzial_dnia(date(2013, 1, 1)) is None


@pytest.mark.django_db
def test_wyczysc_przypisania_wariant_2(wydzial, jednostka):
    Jednostka_Wydzial.objects.create(wydzial=wydzial, jednostka=jednostka)
    Jednostka_Wydzial.objects.wyczysc_przypisania(
        jednostka, date(2012, 1, 1), date(2012, 12, 31)
    )
    assert jednostka.wydzial_dnia(date(2011, 12, 31)) == wydzial
    assert jednostka.wydzial_dnia(date(2012, 1, 1)) is None
    assert jednostka.wydzial_dnia(date(2012, 6, 1)) is None
    assert jednostka.wydzial_dnia(date(2012, 12, 31)) is None
    assert jednostka.wydzial_dnia(date(2013, 1, 1)) == wydzial


@pytest.mark.django_db
def test_wyczysc_przypisania_wariant_3(wydzial, jednostka):
    Jednostka_Wydzial.objects.create(
        wydzial=wydzial, jednostka=jednostka, od=date(2012, 6, 1), do=None
    )
    Jednostka_Wydzial.objects.wyczysc_przypisania(
        jednostka, date(2012, 1, 1), date(2012, 12, 31)
    )
    assert jednostka.wydzial_dnia(date(2011, 12, 31)) is None
    assert jednostka.wydzial_dnia(date(2012, 1, 1)) is None
    assert jednostka.wydzial_dnia(date(2012, 6, 1)) is None
    assert jednostka.wydzial_dnia(date(2012, 12, 31)) is None
    assert jednostka.wydzial_dnia(date(2013, 1, 1)) == wydzial


@pytest.mark.django_db
def test_wyczysc_przypisania_wariant_corner_case_left(wydzial, jednostka):
    Jednostka_Wydzial.objects.create(
        wydzial=wydzial,
        jednostka=jednostka,
        od=date(2011, 12, 31),
        do=date(2012, 12, 31),
    )
    Jednostka_Wydzial.objects.wyczysc_przypisania(
        jednostka, date(2012, 1, 1), date(2012, 12, 31)
    )
    assert jednostka.wydzial_dnia(date(2011, 12, 31)) == wydzial
    assert jednostka.wydzial_dnia(date(2012, 1, 1)) is None



from datetime import timedelta

import pytest
from django.urls import reverse
from django.utils.timezone import localtime


@pytest.mark.django_db
def test_rest_api_autor_detail(client, autor):
    res = client.get(reverse("api_v1:autor-detail", args=(autor.pk,)))
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_autor_list(client, autor):
    res = client.get(reverse("api_v1:autor-list"))
    assert res.status_code == 200


@pytest.mark.django_db
def test_rest_api_autor_filtering_1(api_client, autor):
    czas = localtime(autor.ostatnio_zmieniony).strftime("%Y-%m-%d %H:%M:%S")

    res = api_client.get(
        reverse("api_v1:autor-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 1


@pytest.mark.django_db
def test_rest_api_autor_filtering_2(api_client, autor):
    czas = localtime(autor.ostatnio_zmieniony + timedelta(seconds=1)).strftime(
        "%Y-%m-%d %H:%M:%S"
    )

    res = api_client.get(
        reverse("api_v1:autor-list") + f"?ostatnio_zmieniony_after={czas}"
    )
    assert res.json()["count"] == 0



from import_export.formats.base_formats import XLSX
from tablib.formats import registry


class PrettyXLSX(XLSX):
    TABLIB_MODULE = "prettyxlsx"

    def get_title(self):
        return "prettyxlsx"


registry.register("prettyxlsx", "bpp.admin.xlsx_export._prettyxlsx.PrettyXLSXFormat")



# Generated by Django 3.0.14 on 2021-08-08 23:25

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0023_auto_20210809_0122"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="scientist",
            unique_together={("mongoId", "lastName", "name", "orcid")},
        ),
    ]



# Generated by Django 3.0.14 on 2021-08-13 23:04

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0285_auto_20210814_0103"),
    ]

    operations = [
        migrations.AlterField(
            model_name="praca_doktorska",
            name="jezyk_orig",
            field=models.ForeignKey(
                blank=True,
                help_text="Dla tłumaczeń. Wartość eksportowana do PBN. ",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="bpp.Jezyk",
                verbose_name="Język oryginalny",
            ),
        ),
        migrations.AlterField(
            model_name="praca_habilitacyjna",
            name="jezyk_orig",
            field=models.ForeignKey(
                blank=True,
                help_text="Dla tłumaczeń. Wartość eksportowana do PBN. ",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="bpp.Jezyk",
                verbose_name="Język oryginalny",
            ),
        ),
        migrations.AlterField(
            model_name="wydawnictwo_ciagle",
            name="jezyk_orig",
            field=models.ForeignKey(
                blank=True,
                help_text="Dla tłumaczeń. Wartość eksportowana do PBN. ",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="bpp.Jezyk",
                verbose_name="Język oryginalny",
            ),
        ),
    ]



import math
from decimal import Decimal

from django.utils.functional import cached_property

from bpp.models import Autor


class SlotMixin:
    """Mixin używany przez Wydawnictwo_Zwarte, Wydawnictwo_Ciagle i Patent
    do przeprowadzania kalkulacji na slotach."""

    def __init__(self, original):
        self.original = original

    def wszyscy(self):
        return self.original.autorzy_set.count()

    def autorzy_z_dyscypliny(self, dyscyplina_naukowa, typ_ogolny=None):
        ret = []

        for elem in self.original.autorzy_set.filter(afiliuje=True, przypieta=True):
            if elem.okresl_dyscypline() == dyscyplina_naukowa:
                if typ_ogolny is not None:
                    if elem.typ_odpowiedzialnosci.typ_ogolny != typ_ogolny:
                        continue
                ret.append(elem)
        return ret

    @cached_property
    def dyscypliny(self):
        ret = set()
        for wa in self.original.autorzy_set.all():
            d = wa.okresl_dyscypline()
            if d is None:
                continue
            ret.add(d)
        return ret

    def ma_dyscypline(self, dyscyplina):
        return dyscyplina in self.dyscypliny

    def ensure_autor_rekordu_klass(self, a):
        """
        Jeżeli parametr 'a' to self.original.autor_rekordu_klass, zwraca parametr
        a. Jeżeli parametr 'a' to bpp.models.Autor, znajdź odpowiedni autor_rekordu_klass
        (czyli Wydawnictwo_Ciagle_Autor lub Wydawnictwo_Zwarte_Autor), gdzie ten
        autor występuje.
        """

        if isinstance(a, Autor):
            return self.original.autor_rekordu_klass.objects.get(
                rekord=self.original, autor=a
            )
        return a

    def mnoznik(self, dyscyplina):
        """
        Hook, który wykorzystywany jest później przy obliczaniu punktów
        dla Wydawnictwo_Zwarte
        """
        return 1

    def pkd_dla_autora(self, wca):
        """
        Dzieli PKd (czyli punkty PK dla dyscypliny) przez liczbę k czyli
        przez liczbę wszystkich autorów/redaktorów dla artykułu/książki/rozdziału
        z danej dyscypliny.

        :type wca: bpp.models.WydawnictwoCiagleAutor
        """
        wca = self.ensure_autor_rekordu_klass(wca)

        dyscyplina = wca.okresl_dyscypline()
        azd = len(self.autorzy_z_dyscypliny(dyscyplina))
        if azd == 0:
            return

        pkd = Decimal(self.punkty_pkd(dyscyplina)) * Decimal(self.mnoznik(dyscyplina))
        if pkd is None:
            return

        return pkd / azd

    def slot_dla_autora(self, wca):
        """Normalnie punktację 'slot dla autora z danej dyscypliny' dostajemy w ten
        sposób, że korzystać będziemy z funckji slot_dla_autora_z_dyscypliny.
        Ta funkcja przyjmuje dyscyplinę jako parametr.

        Niekiedy jednak będziemy mieli dostępny obiekt typu Wydawnictwo_Ciagle_Autor,
        niekiedy będziemy mieli samego autora. Żeby uprościć API, funkcja
        'slot_dla_autora' przyjąć może obydwa te parametry (potem upewniając się,
        za pomocą ensure_autor_rekordu_klass, że ma jednak obiekt typu
        Wydawnictwo_Ciagle_Autor). Następnie funkcja określi przypisaną temu
        autorowi dyscyplinę, za pomocą Wydawnictwo_Ciagle_Autor.okresl_dyscypline.
        Następnie, wywoła funkcję slot_dla_autora_z_danej_dyscypliny, podając
        tutaj już dyscyplinę jako parametr.
        """
        wca = self.ensure_autor_rekordu_klass(wca)
        dyscyplina = wca.okresl_dyscypline()
        return self.slot_dla_autora_z_dyscypliny(dyscyplina)

    def liczba_k(self, dyscyplina):
        """Liczba k czyli liczba autorów z dyscypliny (z afiliacją=tak)"""
        if hasattr(self, "_liczba_k_cache"):
            v = self._liczba_k_cache.get(dyscyplina)
            if v is not None:
                return v
        else:
            self._liczba_k_cache = {}

        v = len(self.autorzy_z_dyscypliny(dyscyplina))
        self._liczba_k_cache[dyscyplina] = v
        return v

    def k_przez_m(self, dyscyplina):
        if self.wszyscy() == 0:
            return
        return Decimal(self.liczba_k(dyscyplina) / self.wszyscy())

    def pierwiastek_k_przez_m(self, dyscyplina):
        k_przez_m = self.k_przez_m(dyscyplina)
        if k_przez_m is None:
            return
        return Decimal(math.sqrt(k_przez_m))

    def jeden_przez_wszyscy(self):
        w = self.wszyscy()
        if w == 0:
            return
        return 1 / w



from django.urls import path

from . import views

app_name = "rozbieznosci_dyscyplin"

urlpatterns = [
    path(
        "api/redirect-to-admin/<int:content_type_id>/<int:object_id>/",
        views.RedirectToAdmin.as_view(),
        name="redirect-to-admin",
    )
]



# -*- encoding: utf-8 -*-
import pytest
from django.urls.base import reverse

from bpp.models.cache import Rekord
from django_bpp.selenium_util import wait_for, wait_for_page_load


@pytest.fixture
def multiseek_browser(browser, live_server):
    browser.visit(live_server + reverse("multiseek:index"))
    return browser


def test_wyrzuc(wydawnictwo_zwarte, multiseek_browser, live_server):
    browser = multiseek_browser

    with wait_for_page_load(browser):
        browser.find_by_id("multiseek-szukaj").click()

    browser.execute_script(
        "multiseek.removeFromResults('%s')" % Rekord.objects.all().first().js_safe_pk
    )

    # Poczekaj czy element został skreślony
    wait_for(
        lambda: browser.find_by_css(".multiseek-element")["style"].find("line-through")
        != -1,
    )

    with wait_for_page_load(browser):
        browser.visit(live_server + reverse("multiseek:results"))

    assert "Z zapytania usunięto" in browser.html

    browser.find_by_id("pokaz-jakie").click()

    browser.execute_script(
        "multiseek.removeFromResults('%s')" % Rekord.objects.all().first().js_safe_pk
    )

    # Poczekaj czy element został od-kreślony
    wait_for(
        lambda: browser.find_by_css(".multiseek-element")["style"].find("line-through")
        == -1
    )

    with wait_for_page_load(browser):
        browser.visit(live_server + reverse("multiseek:results"))

    assert "Z zapytania usunięto" not in browser.html


@pytest.mark.django_db
def test_index_copernicus_schowany(multiseek_browser, uczelnia):
    uczelnia.pokazuj_index_copernicus = False
    uczelnia.save()

    multiseek_browser.reload()

    assert "Index Copernicus" not in multiseek_browser.html


@pytest.mark.django_db
def test_index_copernicus_widoczny(multiseek_browser, uczelnia):
    uczelnia.pokazuj_index_copernicus = True
    uczelnia.save()

    multiseek_browser.reload()
    assert "Index Copernicus" in multiseek_browser.html


def test_szukaj(multiseek_browser):
    with wait_for_page_load(multiseek_browser):
        multiseek_browser.find_by_id("multiseek-szukaj").click()
    assert "błąd serwera" not in multiseek_browser.html



from django.contrib.admin import SimpleListFilter

from bpp.admin.filters import SimpleNotNullFilter
from bpp.models import Autor, Wydawca, Zrodlo
from bpp.models.cache import Rekord
from bpp.models.jednostka import Jednostka


class OdpowiednikWBPPFilter(SimpleListFilter):
    db_field_name = None
    klass = Rekord
    fld = "pbn_uid_id"
    title = "Odpowiednik w BPP"
    db_field_name = "pk"
    parameter_name = "odpowiednik"

    def lookups(self, request, model_admin):
        return [("brak", "brak odpowiednika"), ("jest", "ma odpowiednik")]

    def queryset(self, request, queryset):
        v = self.value()

        field = self.db_field_name
        if field is None:
            field = self.parameter_name

        query = {
            field
            + "__in": self.klass.objects.exclude(**{self.fld: None})
            .exclude(**{self.fld: ""})
            .values_list(self.fld, flat=True)
            .distinct()
        }

        if v == "brak":
            return queryset.exclude(**query)
        elif v == "jest":
            return queryset.filter(**query)

        return queryset


class OdpowiednikPublikacjiInstytucjiPublikacjaWBPPFilter(OdpowiednikWBPPFilter):
    db_field_name = "publicationId_id"
    title = "Odpowiednik publikacji w BPP"


OdpowiednikOswiadczeniaInstytucjiPublikacjaWBPPFilter = (
    OdpowiednikPublikacjiInstytucjiPublikacjaWBPPFilter
)


class OdpowiednikPublikacjiInstytucjiAutorWBPPFilter(OdpowiednikWBPPFilter):
    db_field_name = "insPersonId_id"
    title = "Odpowiednik autora w BPP"
    parameter_name = "odpowiednik_autora"
    klass = Autor


class OdpowiednikOswiadczeniaInstytucjiAutorWBPPFilter(
    OdpowiednikPublikacjiInstytucjiAutorWBPPFilter
):
    db_field_name = "personId_id"
    klass = Autor


class OdpowiednikJednostkiWBPPFilter(OdpowiednikWBPPFilter):
    klass = Jednostka


class OdpowiednikWydawcyWBPPFilter(OdpowiednikWBPPFilter):
    klass = Wydawca


class OdpowiednikAutoraWBPPFilter(OdpowiednikWBPPFilter):
    klass = Autor


class OdpowiednikZrodlaWBPPFilter(OdpowiednikWBPPFilter):
    klass = Zrodlo


class MaMNISWIDFilter(SimpleNotNullFilter):
    title = "Rekord ma MNISW ID"
    parameter_name = "mniswId"



# -*- encoding: utf-8 -*-

from django.test import TestCase

# Create your tests here.


class TestCreateTestDb(TestCase):
    def test_create_test_db(self):
        """Pusty testcase - jego uruchomienie zawsze kończy się powodzeniem
        """

        return True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-05 13:44
from __future__ import unicode_literals

from django.db import migrations
from bpp.migration_util import load_custom_sql

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0120_kc_if_3_cyfry'),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0121_rekord_ostatnio_zmieniony_dla")
        ),
    ]
